@@ -1,6 +1,6 @@
+import logging
 import abc
 import numpy as np
-from dipy.utils.six import with_metaclass
 from dipy.core.optimize import Optimizer
 from dipy.align.bundlemin import (_bundle_minimum_distance,
                                   _bundle_minimum_distance_asymmetric,
@@ -10,24 +10,23 @@ from dipy.tracking.streamline import (transform_streamlines,
                                       center_streamlines,
                                       set_number_of_points,
                                       select_random_set_of_streamlines,
-                                      length)
-from dipy.segment.clustering import QuickBundles
+                                      length,
+                                      Streamlines)
+from dipy.segment.clustering import qbx_and_merge
 from dipy.core.geometry import (compose_transformations,
                                 compose_matrix,
                                 decompose_matrix)
-from dipy.utils.six import string_types
 from time import time
 
-MAX_DIST = 1e10
-LOG_MAX_DIST = np.log(MAX_DIST)
-
 DEFAULT_BOUNDS = [(-35, 35), (-35, 35), (-35, 35),
                   (-45, 45), (-45, 45), (-45, 45),
                   (0.6, 1.4), (0.6, 1.4), (0.6, 1.4),
                   (-10, 10), (-10, 10), (-10, 10)]
 
+logger = logging.getLogger(__name__)
 
-class StreamlineDistanceMetric(with_metaclass(abc.ABCMeta, object)):
+
+class StreamlineDistanceMetric(object, metaclass=abc.ABCMeta):
 
     def __init__(self, num_threads=None):
         """ An abstract class for the metric used for streamline registration
@@ -169,10 +168,20 @@ class BundleMinDistanceMatrixMetric(StreamlineDistanceMetric):
 
 class BundleMinDistanceAsymmetricMetric(BundleMinDistanceMetric):
     """ Asymmetric Bundle-based Minimum distance
+
+    This is a cost function that can be used by the
+    StreamlineLinearRegistration class.
+
     """
 
     def distance(self, xopt):
+        """ Distance calculated from this Metric
 
+        Parameters
+        ----------
+        xopt : sequence
+            List of affine parameters as an 1D vector
+        """
         return bundle_min_distance_asymmetric_fast(xopt,
                                                    self.static_centered_pts,
                                                    self.moving_centered_pts,
@@ -210,8 +219,8 @@ class BundleSumDistanceMatrixMetric(BundleMinDistanceMatrixMetric):
 class StreamlineLinearRegistration(object):
 
     def __init__(self, metric=None, x0="rigid", method='L-BFGS-B',
-                 bounds=None, verbose=False, options=None,
-                 evolution=False, num_threads=None):
+                 bounds=None, verbose=False, options=None, evolution=False,
+                 num_threads=None):
         r""" Linear registration of 2 sets of streamlines [Garyfallidis15]_.
 
         Parameters
@@ -265,8 +274,9 @@ class StreamlineLinearRegistration(object):
             That means that we have set the bounds for the three translations
             and three rotation axes (in degrees).
 
-        verbose : bool,
-            If True then information about the optimization is shown.
+        verbose : bool, optional.
+            If True, if True then information about the optimization is shown.
+            Default: False.
 
         options : None or dict,
             Extra options to be used with the selected method.
@@ -282,14 +292,15 @@ class StreamlineLinearRegistration(object):
         References
         ----------
         .. [Garyfallidis15] Garyfallidis et al. "Robust and efficient linear
-            registration of white-matter fascicles in the space of
-            streamlines", NeuroImage, 117, 124--140, 2015
+           registration of white-matter fascicles in the space of streamlines",
+           NeuroImage, 117, 124--140, 2015
+
         .. [Garyfallidis14] Garyfallidis et al., "Direct native-space fiber
-                            bundle alignment for group comparisons", ISMRM,
-                            2014.
+           bundle alignment for group comparisons", ISMRM, 2014.
+
         .. [Garyfallidis17] Garyfallidis et al. Recognition of white matter
-            bundles using local and global streamline-based registration and
-            clustering, Neuroimage, 2017.
+           bundles using local and global streamline-based
+           registration and clustering, Neuroimage, 2017.
         """
 
         self.x0 = self._set_x0(x0)
@@ -377,7 +388,6 @@ class StreamlineLinearRegistration(object):
                             method=self.method,
                             bounds=self.bounds, options=self.options,
                             evolution=self.evolution)
-
         if self.verbose:
             opt.print_summary()
 
@@ -411,7 +421,7 @@ class StreamlineLinearRegistration(object):
                 raise ValueError("Array should have only one dimension")
             return x0
 
-        if isinstance(x0, string_types):
+        if isinstance(x0, str):
 
             if x0.lower() == 'translation':
                 return np.zeros(3)
@@ -512,10 +522,10 @@ def bundle_sum_distance(t, static, moving, num_threads=None):
     Parameters
     -----------
     t : ndarray
-        t is a vector of of affine transformation parameters with
+        t is a vector of affine transformation parameters with
         size at least 6.
-        If size is 6, t is interpreted as translation + rotation.
-        If size is 7, t is interpreted as translation + rotation +
+        If the size is 6, t is interpreted as translation + rotation.
+        If the size is 7, t is interpreted as translation + rotation +
         isotropic scaling.
         If size is 12, t is interpreted as translation + rotation +
         scaling + shearing.
@@ -524,7 +534,7 @@ def bundle_sum_distance(t, static, moving, num_threads=None):
         Static streamlines
 
     moving : list
-        Moving streamlines. These will be transform to align with
+        Moving streamlines. These will be transformed to align with
         the static streamlines
 
     Returns
@@ -548,7 +558,7 @@ def bundle_min_distance(t, static, moving):
     Parameters
     -----------
     t : ndarray
-        t is a vector of of affine transformation parameters with
+        t is a vector of affine transformation parameters with
         size at least 6.
         If size is 6, t is interpreted as translation + rotation.
         If size is 7, t is interpreted as translation + rotation +
@@ -589,10 +599,10 @@ def bundle_min_distance_fast(t, static, moving, block_size, num_threads):
     Parameters
     -----------
     t : array
-        1D array. t is a vector of of affine transformation parameters with
+        1D array. t is a vector of affine transformation parameters with
         size at least 6.
-        If size is 6, t is interpreted as translation + rotation.
-        If size is 7, t is interpreted as translation + rotation +
+        If the size is 6, t is interpreted as translation + rotation.
+        If the size is 7, t is interpreted as translation + rotation +
         isotropic scaling.
         If size is 12, t is interpreted as translation + rotation +
         scaling + shearing.
@@ -652,10 +662,10 @@ def bundle_min_distance_asymmetric_fast(t, static, moving, block_size):
     Parameters
     -----------
     t : array
-        1D array. t is a vector of of affine transformation parameters with
+        1D array. t is a vector of affine transformation parameters with
         size at least 6.
-        If size is 6, t is interpreted as translation + rotation.
-        If size is 7, t is interpreted as translation + rotation +
+        If the size is 6, t is interpreted as translation + rotation.
+        If the size is 7, t is interpreted as translation + rotation +
         isotropic scaling.
         If size is 12, t is interpreted as translation + rotation +
         scaling + shearing.
@@ -693,12 +703,17 @@ def bundle_min_distance_asymmetric_fast(t, static, moving, block_size):
 
 
 def remove_clusters_by_size(clusters, min_size=0):
-    by_size = lambda c: len(c) >= min_size
-    return filter(by_size, clusters)
+    ob = filter(lambda c: len(c) >= min_size, clusters)
+
+    centroids = Streamlines()
+    for cluster in ob:
+        centroids.append(cluster.centroid)
+
+    return centroids
 
 
-def progressive_slr(static, moving, metric, x0, bounds,
-                    method='L-BFGS-B', verbose=True, num_threads=None):
+def progressive_slr(static, moving, metric, x0, bounds, method='L-BFGS-B',
+                    verbose=False, num_threads=None):
     """ Progressive SLR
 
     This is an utility function that allows for example to do affine
@@ -706,7 +721,7 @@ def progressive_slr(static, moving, metric, x0, bounds,
     [Garyfallidis15]_ by starting with translation first, then rigid,
     then similarity, scaling and finally affine.
 
-    Similarly, if for example you want to perform rigid then you start with
+    Similarly, if for example, you want to perform rigid then you start with
     translation first. This progressive strategy can helps with finding the
     optimal parameters of the final transformation.
 
@@ -723,8 +738,8 @@ def progressive_slr(static, moving, metric, x0, bounds,
         for example.
     method : string
         L_BFGS_B' or 'Powell' optimizers can be used. Default is 'L_BFGS_B'.
-    verbose : bool
-        If True show messages in stdout (default True).
+    verbose :  bool, optional.
+        If True, log messages. Default:
     num_threads : int
         Number of threads. If None (default) then all available threads
         will be used. Only metrics using OpenMP will use this variable.
@@ -735,15 +750,13 @@ def progressive_slr(static, moving, metric, x0, bounds,
         registration of white-matter fascicles in the space of streamlines",
         NeuroImage, 117, 124--140, 2015
     """
-
     if verbose:
-        print('Progressive Registration is Enabled')
+        logger.info('Progressive Registration is Enabled')
 
     if x0 == 'translation' or x0 == 'rigid' or \
        x0 == 'similarity' or x0 == 'scaling' or x0 == 'affine':
-
         if verbose:
-            print(' Translation  (3 parameters)...')
+            logger.info(' Translation  (3 parameters)...')
         slr_t = StreamlineLinearRegistration(metric=metric,
                                              x0='translation',
                                              bounds=bounds[:3],
@@ -757,9 +770,8 @@ def progressive_slr(static, moving, metric, x0, bounds,
         x_translation = slm_t.xopt
         x = np.zeros(6)
         x[:3] = x_translation
-
         if verbose:
-            print(' Rigid  (6 parameters) ...')
+            logger.info(' Rigid  (6 parameters) ...')
         slr_r = StreamlineLinearRegistration(metric=metric,
                                              x0=x,
                                              bounds=bounds[:6],
@@ -772,9 +784,8 @@ def progressive_slr(static, moving, metric, x0, bounds,
         x = np.zeros(7)
         x[:6] = x_rigid
         x[6] = 1.
-
         if verbose:
-            print(' Similarity (7 parameters) ...')
+            logger.info(' Similarity (7 parameters) ...')
         slr_s = StreamlineLinearRegistration(metric=metric,
                                              x0=x,
                                              bounds=bounds[:7],
@@ -787,9 +798,8 @@ def progressive_slr(static, moving, metric, x0, bounds,
         x = np.zeros(9)
         x[:6] = x_similarity[:6]
         x[6:] = np.array((x_similarity[6],) * 3)
-
         if verbose:
-            print(' Scaling (9 parameters) ...')
+            logger.info(' Scaling (9 parameters) ...')
 
         slr_c = StreamlineLinearRegistration(metric=metric,
                                              x0=x,
@@ -803,9 +813,8 @@ def progressive_slr(static, moving, metric, x0, bounds,
         x = np.zeros(12)
         x[:9] = x_scaling[:9]
         x[9:] = np.zeros(3)
-
         if verbose:
-            print(' Affine (12 parameters) ...')
+            logger.info(' Affine (12 parameters) ...')
 
         slr_a = StreamlineLinearRegistration(metric=metric,
                                              x0=x,
@@ -829,41 +838,58 @@ def progressive_slr(static, moving, metric, x0, bounds,
     return slm
 
 
-def slr_with_qb(static, moving,
-                x0='affine',
-                rm_small_clusters=50,
-                maxiter=100,
-                select_random=None,
-                verbose=False,
-                greater_than=50,
-                less_than=250,
-                qb_thr=15,
-                nb_pts=20,
-                progressive=True, num_threads=None):
+def slr_with_qbx(static, moving,
+                 x0='affine',
+                 rm_small_clusters=50,
+                 maxiter=100,
+                 select_random=None,
+                 verbose=False,
+                 greater_than=50,
+                 less_than=250,
+                 qbx_thr=[40, 30, 20, 15],
+                 nb_pts=20,
+                 progressive=True, rng=None, num_threads=None):
     """ Utility function for registering large tractograms.
 
-    For efficiency we apply the registration on cluster centroids and remove
+    For efficiency, we apply the registration on cluster centroids and remove
     small clusters.
 
     Parameters
     ----------
     static : Streamlines
     moving : Streamlines
-    x0 : str
+
+    x0 : str, optional.
         rigid, similarity or affine transformation model (default affine)
 
-    rm_small_clusters : int
+    rm_small_clusters : int, optional
         Remove clusters that have less than `rm_small_clusters` (default 50)
 
-    verbose : bool,
-        If True then information about the optimization is shown.
-
-    select_random : int
-        If not None select a random number of streamlines to apply clustering
+    select_random : int, optional.
+        If not, None selects a random number of streamlines to apply clustering
         Default None.
 
-    options : None or dict,
-        Extra options to be used with the selected method.
+    verbose : bool, optional
+        If True, logs information about optimization. Default: False
+
+    greater_than : int, optional
+            Keep streamlines that have length greater than
+            this value (default 50)
+
+    less_than : int, optional
+            Keep streamlines have length less than this value (default 250)
+
+    qbx_thr : variable int
+            Thresholds for QuickBundlesX (default [40, 30, 20, 15])
+
+    np_pts : int, optional
+            Number of points for discretizing each streamline (default 20)
+
+    progressive : boolean, optional
+            (default True)
+
+    rng : RandomState
+        If None creates RandomState in function.
 
     num_threads : int
         Number of threads. If None (default) then all available threads
@@ -872,23 +898,26 @@ def slr_with_qb(static, moving,
     Notes
     -----
     The order of operations is the following. First short or long streamlines
-    are removed. Second the tractogram or a random selection of the tractogram
+    are removed. Second, the tractogram or a random selection of the tractogram
     is clustered with QuickBundles. Then SLR [Garyfallidis15]_ is applied.
 
     References
     ----------
     .. [Garyfallidis15] Garyfallidis et al. "Robust and efficient linear
-            registration of white-matter fascicles in the space of streamlines"
-            , NeuroImage, 117, 124--140, 2015
+    registration of white-matter fascicles in the space of streamlines",
+    NeuroImage, 117, 124--140, 2015
     .. [Garyfallidis14] Garyfallidis et al., "Direct native-space fiber
             bundle alignment for group comparisons", ISMRM, 2014.
     .. [Garyfallidis17] Garyfallidis et al. Recognition of white matter
-            bundles using local and global streamline-based registration and
-            clustering, Neuroimage, 2017.
+    bundles using local and global streamline-based registration and
+    clustering, Neuroimage, 2017.
     """
+    if rng is None:
+        rng = np.random.RandomState()
+
     if verbose:
-        print('Static streamlines size {}'.format(len(static)))
-        print('Moving streamlines size {}'.format(len(moving)))
+        logger.info('Static streamlines size {}'.format(len(static)))
+        logger.info('Moving streamlines size {}'.format(len(moving)))
 
     def check_range(streamline, gt=greater_than, lt=less_than):
 
@@ -897,42 +926,43 @@ def slr_with_qb(static, moving,
         else:
             return False
 
-    # TODO change this to the new Streamlines API
-    streamlines1 = [s for s in static if check_range(s)]
-    streamlines2 = [s for s in moving if check_range(s)]
-
+    streamlines1 = Streamlines(static[np.array([check_range(s)
+                                                for s in static])])
+    streamlines2 = Streamlines(moving[np.array([check_range(s)
+                                                for s in moving])])
     if verbose:
-
-        print('Static streamlines after length reduction {}'
-              .format(len(streamlines1)))
-        print('Moving streamlines after length reduction {}'
-              .format(len(streamlines2)))
+        logger.info('Static streamlines after length reduction {}'
+                    .format(len(streamlines1)))
+        logger.info('Moving streamlines after length reduction {}'
+                    .format(len(streamlines2)))
 
     if select_random is not None:
         rstreamlines1 = select_random_set_of_streamlines(streamlines1,
-                                                         select_random)
+                                                         select_random,
+                                                         rng=rng)
     else:
         rstreamlines1 = streamlines1
 
     rstreamlines1 = set_number_of_points(rstreamlines1, nb_pts)
-    qb1 = QuickBundles(threshold=qb_thr)
-    rstreamlines1 = [s.astype('f4') for s in rstreamlines1]
-    cluster_map1 = qb1.cluster(rstreamlines1)
-    clusters1 = remove_clusters_by_size(cluster_map1, rm_small_clusters)
-    qb_centroids1 = [cluster.centroid for cluster in clusters1]
+
+    rstreamlines1._data.astype('f4')
+
+    cluster_map1 = qbx_and_merge(rstreamlines1, thresholds=qbx_thr, rng=rng)
+    qb_centroids1 = remove_clusters_by_size(cluster_map1, rm_small_clusters)
 
     if select_random is not None:
         rstreamlines2 = select_random_set_of_streamlines(streamlines2,
-                                                         select_random)
+                                                         select_random,
+                                                         rng=rng)
     else:
         rstreamlines2 = streamlines2
 
     rstreamlines2 = set_number_of_points(rstreamlines2, nb_pts)
-    qb2 = QuickBundles(threshold=qb_thr)
-    rstreamlines2 = [s.astype('f4') for s in rstreamlines2]
-    cluster_map2 = qb2.cluster(rstreamlines2)
-    clusters2 = remove_clusters_by_size(cluster_map2, rm_small_clusters)
-    qb_centroids2 = [cluster.centroid for cluster in clusters2]
+    rstreamlines2._data.astype('f4')
+
+    cluster_map2 = qbx_and_merge(rstreamlines2, thresholds=qbx_thr, rng=rng)
+
+    qb_centroids2 = remove_clusters_by_size(cluster_map2, rm_small_clusters)
 
     if verbose:
         t = time()
@@ -950,12 +980,12 @@ def slr_with_qb(static, moving,
                               bounds=bounds, num_threads=num_threads)
 
     if verbose:
-        print('QB static centroids size %d' % len(qb_centroids1,))
-        print('QB moving centroids size %d' % len(qb_centroids2,))
+        logger.info('QB static centroids size %d' % len(qb_centroids1,))
+        logger.info('QB moving centroids size %d' % len(qb_centroids2,))
         duration = time() - t
-        print('SLR finished in  %0.3f seconds.' % (duration,))
+        logger.info('SLR finished in  %0.3f seconds.' % (duration,))
         if slm.iterations is not None:
-            print('SLR iterations: %d ' % (slm.iterations,))
+            logger.info('SLR iterations: %d ' % (slm.iterations,))
 
     moved = slm.transform(moving)
 
@@ -966,8 +996,8 @@ def slr_with_qb(static, moving,
 # SLR on QuickBundles centroids and some thresholding see
 # Garyfallidis et al. Recognition of white matter
 # bundles using local and global streamline-based registration and
-# clustering, Neuroimage, 2017.
-whole_brain_slr = slr_with_qb
+# clustering, NeuroImage, 2017.
+whole_brain_slr = slr_with_qbx
 
 
 def _threshold(x, th):
@@ -980,13 +1010,13 @@ def compose_matrix44(t, dtype=np.double):
     Parameters
     -----------
     t : ndarray
-        This is a 1D vector of of affine transformation parameters with
+        This is a 1D vector of affine transformation parameters with
         size at least 3.
-        If size is 3, t is interpreted as translation.
-        If size is 6, t is interpreted as translation + rotation.
-        If size is 7, t is interpreted as translation + rotation +
+        If the size is 3, t is interpreted as translation.
+        If the size is 6, t is interpreted as translation + rotation.
+        If the size is 7, t is interpreted as translation + rotation +
         isotropic scaling.
-        If size is 9, t is interpreted as translation + rotation +
+        If the size is 9, t is interpreted as translation + rotation +
         anisotropic scaling.
         If size is 12, t is interpreted as translation + rotation +
         scaling + shearing.
@@ -1004,6 +1034,7 @@ def compose_matrix44(t, dtype=np.double):
     if size not in [3, 6, 7, 9, 12]:
         raise ValueError('Accepted number of parameters is 3, 6, 7, 9 and 12')
 
+    MAX_DIST = 1e10
     scale, shear, angles, translate = (None, ) * 4
     translate = _threshold(t[0:3], MAX_DIST)
     if size in [6, 7, 9, 12]:
@@ -1027,7 +1058,7 @@ def decompose_matrix44(mat, size=12):
     mat : array
         Homogeneous 4x4 transformation matrix
     size : int
-        Size of output vector. 3, for translation, 6 for rigid,
+        Size of the output vector. 3, for translation, 6 for rigid,
         7 for similarity, 9 for scaling and 12 for affine. Default is 12.
 
     Returns
