@@ -8,7 +8,6 @@ from numpy.testing import (assert_array_equal,
                            assert_equal,
                            assert_raises)
 from dipy.core import geometry as geometry
-from dipy.data import get_data
 from dipy.viz import regtools as rt
 from dipy.align import floating
 from dipy.align import vector_fields as vf
@@ -46,10 +45,10 @@ def test_transform_centers_of_mass_3d():
     np.random.seed(1246592)
     shape = (64, 64, 64)
     rm = 8
-    sp = vf.create_sphere(shape[0] // 2, shape[1] // 2, shape[2] // 2, rm)
+    sph = vf.create_sphere(shape[0] // 2, shape[1] // 2, shape[2] // 2, rm)
     moving = np.zeros(shape)
     # The center of mass will be (16, 16, 16), in image coordinates
-    moving[:shape[0] // 2, :shape[1] // 2, :shape[2] // 2] = sp[...]
+    moving[:shape[0] // 2, :shape[1] // 2, :shape[2] // 2] = sph[...]
 
     rs = 16
     # The center of mass will be (32, 32, 32), in image coordinates
@@ -186,12 +185,10 @@ def test_affreg_all_transforms():
     # Test affine registration using all transforms with typical settings
 
     # Make sure dictionary entries are processed in the same order regardless
-    # of the platform.
-    # Otherwise any random numbers drawn within the loop would make
-    # the test non-deterministic even if we fix the seed before the loop.
-    # Right now, this test does not draw any samples,
-    # but we still sort the entries
-    # to prevent future related failures.
+    # of the platform. Otherwise any random numbers drawn within the loop would
+    # make the test non-deterministic even if we fix the seed before the loop.
+    # Right now, this test does not draw any samples, but we still sort the
+    # entries to prevent future related failures.
     for ttype in sorted(factors):
         dim = ttype[1]
         if dim == 2:
@@ -200,9 +197,14 @@ def test_affreg_all_transforms():
             nslices = 45
         factor = factors[ttype][0]
         sampling_pc = factors[ttype][1]
-        transform = regtransforms[ttype]
-        static, moving, static_grid2world, moving_grid2world, smask, mmask, T = \
-            setup_random_transform(transform, factor, nslices, 1.0)
+        trans = regtransforms[ttype]
+        # Shorthand:
+        srt = setup_random_transform
+        static, moving, static_g2w, moving_g2w, smask, mmask, T = srt(
+                                                                      trans,
+                                                                      factor,
+                                                                      nslices,
+                                                                      1.0)
         # Sum of absolute differences
         start_sad = np.abs(static - moving).sum()
         metric = imaffine.MutualInformationMetric(32, sampling_pc)
@@ -213,9 +215,9 @@ def test_affreg_all_transforms():
                                              'L-BFGS-B',
                                              None,
                                              options=None)
-        x0 = transform.get_identity_parameters()
-        affine_map = affreg.optimize(static, moving, transform, x0,
-                                     static_grid2world, moving_grid2world)
+        x0 = trans.get_identity_parameters()
+        affine_map = affreg.optimize(static, moving, trans, x0,
+                                     static_g2w, moving_g2w)
         transformed = affine_map.transform(moving)
         # Sum of absolute differences
         end_sad = np.abs(static - transformed).sum()
@@ -319,7 +321,7 @@ def test_mi_gradient():
         # Compute the gradient with the implementation under test
         actual = mi_metric.gradient(theta)
 
-        # Compute the gradient using finite-diferences
+        # Compute the gradient using finite-differences
         n = transform.get_number_of_parameters()
         expected = np.empty(n, dtype=np.float64)
 
@@ -470,7 +472,7 @@ def test_affine_map():
                 # compatibility with previous versions
                 assert_array_equal(affine, affine_map.affine)
                 # new getter
-                new_copy_affine = affine_map.get_affine()
+                new_copy_affine = affine_map.affine
                 # value must be the same
                 assert_array_equal(affine, new_copy_affine)
                 # but not its reference
@@ -512,12 +514,12 @@ def test_affine_map():
             aff_map = AffineMap(affine_mat)
             if affine_mat is None:
                 continue
-            bad_aug = aff_map.get_affine()
+            bad_aug = aff_map.affine
             # no zeros in the first n-1 columns on last row
             bad_aug[-1,:] = 1
             assert_raises(AffineInvalidValuesError, AffineMap, bad_aug)
 
-            bad_aug = aff_map.get_affine()
+            bad_aug = aff_map.affine
             bad_aug[-1, -1] = 0  # lower right not 1
             assert_raises(AffineInvalidValuesError, AffineMap, bad_aug)
 
