@@ -1,14 +1,14 @@
-""" Tools for using spherical harmonic models to fit diffusion data
+"""Tools for using spherical harmonic models to fit diffusion data.
 
 References
 ----------
-Aganj, I., et. al. 2009. ODF Reconstruction in Q-Ball Imaging With Solid
+Aganj, I., et al. 2009. ODF Reconstruction in Q-Ball Imaging With Solid
     Angle Consideration.
-Descoteaux, M., et. al. 2007. Regularized, fast, and robust analytical
+Descoteaux, M., et al. 2007. Regularized, fast, and robust analytical
     Q-ball imaging.
-Tristan-Vega, A., et. al. 2010. A new methodology for estimation of fiber
+Tristan-Vega, A., et al. 2010. A new methodology for estimation of fiber
     populations in white matter of the brain with Funk-Radon transform.
-Tristan-Vega, A., et. al. 2009. Estimation of fiber orientation probability
+Tristan-Vega, A., et al. 2009. Estimation of fiber orientation probability
     density functions in high angular resolution diffusion imaging.
 
 
@@ -25,8 +25,8 @@ where data is Y.T and sh_coef is x.T.
 """
 
 import numpy as np
-from numpy import concatenate, diag, diff, empty, eye, sqrt, unique, dot
-from numpy.linalg import pinv, svd
+import scipy.special as sps
+
 from numpy.random import randint
 
 from dipy.reconst.odf import OdfModel, OdfFit
@@ -34,16 +34,6 @@ from dipy.core.geometry import cart2sphere
 from dipy.core.onetime import auto_attr
 from dipy.reconst.cache import Cache
 
-from distutils.version import LooseVersion
-import scipy
-from scipy.special import lpn, lpmv, gammaln
-
-if LooseVersion(scipy.version.short_version) >= LooseVersion('0.15.0'):
-    SCIPY_15_PLUS = True
-    import scipy.special as sps
-else:
-    SCIPY_15_PLUS = False
-
 
 def _copydoc(obj):
     def bandit(f):
@@ -157,19 +147,8 @@ def gen_dirac(m, n, theta, phi):
     return real_sph_harm(m, n, theta, phi)
 
 
-def spherical_harmonics(m, n, theta, phi):
-    x = np.cos(phi)
-    val = lpmv(m, n, x).astype(complex)
-    val *= np.sqrt((2 * n + 1) / 4.0 / np.pi)
-    val *= np.exp(0.5 * (gammaln(n - m + 1) - gammaln(n + m + 1)))
-    val = val * np.exp(1j * m * theta)
-    return val
-
-if SCIPY_15_PLUS:
-    def spherical_harmonics(m, n, theta, phi):
-        return sps.sph_harm(m, n, theta, phi, dtype=complex)
-
-spherical_harmonics.__doc__ = r""" Compute spherical harmonics
+def spherical_harmonics(m, n, theta, phi, use_scipy=True):
+    """Compute spherical harmonics.
 
     This may take scalar or array arguments. The inputs will be broadcasted
     against each other.
@@ -184,6 +163,8 @@ spherical_harmonics.__doc__ = r""" Compute spherical harmonics
         The azimuthal (longitudinal) coordinate.
     phi : float [0, pi]
         The polar (colatitudinal) coordinate.
+    use_scipy : bool
+        if True, use scipy implementation. default(True)
 
     Returns
     -------
@@ -195,7 +176,17 @@ spherical_harmonics.__doc__ = r""" Compute spherical harmonics
     This is a faster implementation of scipy.special.sph_harm for
     scipy version < 0.15.0. For scipy 0.15 and onwards, we use the scipy
     implementation of the function
+
     """
+    if use_scipy:
+        return sps.sph_harm(m, n, theta, phi, dtype=complex)
+
+    x = np.cos(phi)
+    val = sps.lpmv(m, n, x).astype(complex)
+    val *= np.sqrt((2 * n + 1) / 4.0 / np.pi)
+    val *= np.exp(0.5 * (sps.gammaln(n - m + 1) - sps.gammaln(n + m + 1)))
+    val = val * np.exp(1j * m * theta)
+    return val
 
 
 def real_sph_harm(m, n, theta, phi):
@@ -241,8 +232,8 @@ def real_sph_harm(m, n, theta, phi):
 
 def real_sym_sh_mrtrix(sh_order, theta, phi):
     """
-    Compute real spherical harmonics as in mrtrix, where the real harmonic
-    $Y^m_n$ is defined to be::
+    Compute real symmetric spherical harmonics as in Tournier 2007 [2]_, where
+    the real harmonic $Y^m_n$ is defined to be::
 
         Real($Y^m_n$)       if m > 0
         $Y^0_n$             if m = 0
@@ -264,13 +255,24 @@ def real_sym_sh_mrtrix(sh_order, theta, phi):
     --------
     y_mn : real float
         The real harmonic $Y^m_n$ sampled at `theta` and `phi` as
-        implemented in mrtrix.  Warning: the basis is Tournier et al
-        2004 and 2007 is slightly different.
+        implemented in mrtrix. Warning: the basis is Tournier et al.
+        2007 [2]_; 2004 [1]_ is slightly different.
     m : array
         The order of the harmonics.
     n : array
         The degree of the harmonics.
 
+    References
+    ----------
+    .. [1] Tournier J.D., Calamante F., Gadian D.G. and Connelly A.
+           Direct estimation of the fibre orientation density function from
+           diffusion-weighted MRI data using spherical deconvolution.
+           NeuroImage. 2004;23:1176-1185.
+    .. [2] Tournier J.D., Calamante F. and Connelly A. Robust determination
+           of the fibre orientation distribution in diffusion MRI:
+           Non-negativity constrained super-resolved spherical deconvolution.
+           NeuroImage. 2007;35(4):1459-1472.
+
     """
     m, n = sph_harm_ind_list(sh_order)
     phi = np.reshape(phi, [-1, 1])
@@ -282,13 +284,67 @@ def real_sym_sh_mrtrix(sh_order, theta, phi):
     return real_sh, m, n
 
 
+def real_full_sh_mrtrix(sh_order, theta, phi):
+    """
+    Compute real spherical harmonics as in Tournier 2007 [2]_, where the real
+    harmonic $Y^m_n$ is defined to be::
+
+        Real($Y^m_n$)       if m > 0
+        $Y^0_n$             if m = 0
+        Imag($Y^|m|_n$)     if m < 0
+
+    This may take scalar or array arguments. The inputs will be broadcasted
+    against each other.
+
+    Parameters
+    -----------
+    sh_order : int
+        The maximum degree or the spherical harmonic basis.
+    theta : float [0, pi]
+        The polar (colatitudinal) coordinate.
+    phi : float [0, 2*pi]
+        The azimuthal (longitudinal) coordinate.
+
+    Returns
+    --------
+    y_mn : real float
+        The real harmonic $Y^m_n$ sampled at `theta` and `phi` as
+        implemented in mrtrix. Warning: the basis is Tournier et al.
+        2007 [2]_; 2004 [1]_ is slightly different.
+    m : array
+        The order of the harmonics.
+    n : array
+        The degree of the harmonics.
+
+    References
+    ----------
+    .. [1] Tournier J.D., Calamante F., Gadian D.G. and Connelly A.
+           Direct estimation of the fibre orientation density function from
+           diffusion-weighted MRI data using spherical deconvolution.
+           NeuroImage. 2004;23:1176-1185.
+    .. [2] Tournier J.D., Calamante F. and Connelly A. Robust determination
+           of the fibre orientation distribution in diffusion MRI:
+           Non-negativity constrained super-resolved spherical deconvolution.
+           NeuroImage. 2007;35(4):1459-1472.
+
+    """
+    m, n = sph_harm_full_ind_list(sh_order)
+    phi = np.reshape(phi, [-1, 1])
+    theta = np.reshape(theta, [-1, 1])
+
+    m = -m
+    real_sh = real_sph_harm(m, n, theta, phi)
+    real_sh /= np.where(m == 0, 1., np.sqrt(2))
+    return real_sh, m, n
+
+
 def real_sym_sh_basis(sh_order, theta, phi):
     """Samples a real symmetric spherical harmonic basis at point on the sphere
 
     Samples the basis functions up to order `sh_order` at points on the sphere
     given by `theta` and `phi`. The basis functions are defined here the same
-    way as in fibernavigator [1]_ where the real harmonic $Y^m_n$ is defined to
-    be:
+    way as in Descoteaux et al. 2007 [1]_ where the real harmonic $Y^m_n$ is
+    defined to be:
 
         Imag($Y^m_n$) * sqrt(2)     if m > 0
         $Y^0_n$                     if m = 0
@@ -317,7 +373,9 @@ def real_sym_sh_basis(sh_order, theta, phi):
 
     References
     ----------
-    .. [1] https://github.com/scilus/fibernavigator
+    .. [1] Descoteaux, M., Angelino, E., Fitzgibbons, S. and Deriche, R.
+           Regularized, Fast, and Robust Analytical Q-ball Imaging.
+           Magn. Reson. Med. 2007;58:497-510.
 
     """
     m, n = sph_harm_ind_list(sh_order)
@@ -328,9 +386,59 @@ def real_sym_sh_basis(sh_order, theta, phi):
     return real_sh, m, n
 
 
+def real_full_sh_basis(sh_order, theta, phi):
+    """Samples a real spherical harmonic basis at point on the sphere
+
+    Samples the basis functions up to order `sh_order` at points on the sphere
+    given by `theta` and `phi`. The basis functions are defined here the same
+    way as in Descoteaux et al. 2007 [1]_ where the real harmonic $Y^m_n$ is
+    defined to be:
+
+        Imag($Y^m_n$) * sqrt(2)     if m > 0
+        $Y^0_n$                     if m = 0
+        Real($Y^|m|_n$) * sqrt(2)   if m < 0
+
+    This may take scalar or array arguments. The inputs will be broadcasted
+    against each other.
+
+    Parameters
+    -----------
+    sh_order : int
+        int > 0, max spherical harmonic degree
+    theta : float [0, 2*pi]
+        The azimuthal (longitudinal) coordinate.
+    phi : float [0, pi]
+        The polar (colatitudinal) coordinate.
+
+    Returns
+    --------
+    y_mn : real float
+        The real harmonic $Y^m_n$ sampled at `theta` and `phi`
+    m : array
+        The order of the harmonics.
+    n : array
+        The degree of the harmonics.
+
+    References
+    ----------
+    .. [1] Descoteaux, M., Angelino, E., Fitzgibbons, S. and Deriche, R.
+           Regularized, Fast, and Robust Analytical Q-ball Imaging.
+           Magn. Reson. Med. 2007;58:497-510.
+
+    """
+    m, n = sph_harm_full_ind_list(sh_order)
+    phi = np.reshape(phi, [-1, 1])
+    theta = np.reshape(theta, [-1, 1])
+
+    real_sh = real_sph_harm(m, n, theta, phi)
+    return real_sh, m, n
+
+
 sph_harm_lookup = {None: real_sym_sh_basis,
-                   "mrtrix": real_sym_sh_mrtrix,
-                   "fibernav": real_sym_sh_basis}
+                   "tournier07": real_sym_sh_mrtrix,
+                   "tournier07_full": real_full_sh_mrtrix,
+                   "descoteaux07": real_sym_sh_basis,
+                   "descoteaux07_full": real_full_sh_basis}
 
 
 def sph_harm_ind_list(sh_order):
@@ -364,7 +472,7 @@ def sph_harm_ind_list(sh_order):
 
     ncoef = int((sh_order + 2) * (sh_order + 1) // 2)
     offset = 0
-    m_list = empty(ncoef, 'int')
+    m_list = np.empty(ncoef, 'int')
     for ii in n_range:
         m_list[offset:offset + 2 * ii + 1] = np.arange(-ii, ii + 1)
         offset = offset + 2 * ii + 1
@@ -373,10 +481,64 @@ def sph_harm_ind_list(sh_order):
     return (m_list, n_list)
 
 
-def order_from_ncoef(ncoef):
+def sph_harm_full_ind_list(sh_order):
+    """
+    Returns the degree (n) and order (m) of all the spherical
+    harmonics of degree less then or equal to `sh_order`. The results, `m_list`
+    and `n_list` are kx1 arrays, where k depends on sh_order. They can be
+    passed to :func:`real_sph_harm`.
+
+    Parameters
+    ----------
+    sh_order : int
+        int > 0, max degree to return
+
+    Returns
+    -------
+    m_list : array
+        orders of spherical harmonics
+    n_list : array
+        degrees of spherical harmonics
+
+    See also
+    --------
+    real_sph_harm
+    """
+    n_range = np.arange(0, sh_order + 1, dtype=int)
+    n_list = np.repeat(n_range, n_range * 2 + 1)
+
+    ncoef = int((sh_order + 1) * (sh_order + 1))
+    offset = 0
+    m_list = np.empty(ncoef, 'int')
+    for ii in n_range:
+        m_list[offset:offset + 2 * ii + 1] = np.arange(-ii, ii + 1)
+        offset = offset + 2 * ii + 1
+
+    # makes the arrays ncoef by 1, allows for easy broadcasting later in code
+    return (m_list, n_list)
+
+
+def order_from_ncoef(ncoef, is_full_basis=False):
     """
     Given a number n of coefficients, calculate back the sh_order
+
+    Parameters
+    ----------
+    ncoef: int
+        number of coefficients
+    is_full_basis: bool, optional
+        True when coefficients are for a full SH basis
+
+    Returns
+    -------
+    sh_order: int
+        maximum order of SH basis
     """
+    if is_full_basis:
+        # Solve the equation :
+        # ncoef = (sh_order + 1) * (sh_order + 1)
+        return int(np.sqrt(ncoef) - 1)
+
     # Solve the quadratic equation derived from :
     # ncoef = (sh_order + 2) * (sh_order + 1) / 2
     return int(-3 + np.sqrt(9 - 4 * (2-2*ncoef)))/2
@@ -406,8 +568,8 @@ def smooth_pinv(B, L):
     product.
 
     """
-    L = diag(L)
-    inv = pinv(concatenate((B, L)))
+    L = np.diag(L)
+    inv = np.linalg.pinv(np.concatenate((B, L)))
     return inv[:, :len(B)]
 
 
@@ -423,7 +585,7 @@ def lazy_index(index):
         index = index.nonzero()[0]
     if len(index) == 1:
         return slice(index[0], index[0] + 1)
-    step = unique(diff(index))
+    step = np.unique(np.diff(index))
     if len(step) != 1 or step[0] == 0:
         return index
     else:
@@ -491,12 +653,12 @@ class SphHarmModel(OdfModel, Cache):
 
 class QballBaseModel(SphHarmModel):
     """To be subclassed by Qball type models."""
-    def __init__(self, gtab, sh_order, smooth=0.006, min_signal=1.,
+    def __init__(self, gtab, sh_order, smooth=0.006, min_signal=1e-5,
                  assume_normed=False):
         """Creates a model that can be used to fit or sample diffusion data
 
-        Arguments
-        ---------
+        Parameters
+        ------------
         gtab : GradientTable
             Diffusion gradients used to acquire data
         sh_order : even int >= 0
@@ -526,7 +688,7 @@ class QballBaseModel(SphHarmModel):
         r, theta, phi = cart2sphere(x, y, z)
         B, m, n = real_sym_sh_basis(sh_order, theta[:, None], phi[:, None])
         L = -n * (n + 1)
-        legendre0 = lpn(sh_order, 0)[0]
+        legendre0 = sps.lpn(sh_order, 0)[0]
         F = legendre0[n]
         self.sh_order = sh_order
         self.B = B
@@ -600,11 +762,11 @@ class SphHarmFit(OdfFit):
 
         """
         B = self.model.sampling_matrix(sphere)
-        return dot(self._shm_coef, B.T)
+        return np.dot(self.shm_coeff, B.T)
 
     @auto_attr
     def gfa(self):
-        return _gfa_sh(self._shm_coef, 0)
+        return _gfa_sh(self.shm_coeff, 0)
 
     @property
     def shm_coeff(self):
@@ -632,7 +794,7 @@ class SphHarmFit(OdfFit):
         if not hasattr(self.model, 'predict'):
             msg = "This model does not have prediction implemented yet"
             raise NotImplementedError(msg)
-        return self.model.predict(self.shm_coeff, gtab, S0)
+        return self.model.predict(self._shm_coef, gtab, S0)
 
 
 class CsaOdfModel(QballBaseModel):
@@ -640,7 +802,7 @@ class CsaOdfModel(QballBaseModel):
 
     References
     ----------
-    .. [1] Aganj, I., et. al. 2009. ODF Reconstruction in Q-Ball Imaging With
+    .. [1] Aganj, I., et al. 2009. ODF Reconstruction in Q-Ball Imaging With
            Solid Angle Consideration.
     """
     min = .001
@@ -650,7 +812,7 @@ class CsaOdfModel(QballBaseModel):
     def _set_fit_matrix(self, B, L, F, smooth):
         """The fit matrix, is used by fit_coefficients to return the
         coefficients of the odf"""
-        invB = smooth_pinv(B, sqrt(smooth) * L)
+        invB = smooth_pinv(B, np.sqrt(smooth) * L)
         L = L[:, None]
         F = F[:, None]
         self._fit_matrix = (F * L) / (8 * np.pi) * invB
@@ -660,7 +822,7 @@ class CsaOdfModel(QballBaseModel):
         data = data[..., self._where_dwi]
         data = data.clip(self.min, self.max)
         loglog_data = np.log(-np.log(data))
-        sh_coef = dot(loglog_data, self._fit_matrix.T)
+        sh_coef = np.dot(loglog_data, self._fit_matrix.T)
         sh_coef[..., 0] = self._n0_const
         return sh_coef
 
@@ -671,15 +833,15 @@ class OpdtModel(QballBaseModel):
 
     References
     ----------
-    .. [1] Tristan-Vega, A., et. al. 2010. A new methodology for estimation of
+    .. [1] Tristan-Vega, A., et al. 2010. A new methodology for estimation of
            fiber populations in white matter of the brain with Funk-Radon
            transform.
-    .. [2] Tristan-Vega, A., et. al. 2009. Estimation of fiber orientation
+    .. [2] Tristan-Vega, A., et al. 2009. Estimation of fiber orientation
            probability density functions in high angular resolution diffusion
            imaging.
     """
     def _set_fit_matrix(self, B, L, F, smooth):
-        invB = smooth_pinv(B, sqrt(smooth) * L)
+        invB = smooth_pinv(B, np.sqrt(smooth) * L)
         L = L[:, None]
         F = F[:, None]
         delta_b = F * L * invB
@@ -695,7 +857,8 @@ class OpdtModel(QballBaseModel):
 def _slowadc_formula(data, delta_b, delta_q):
     """formula used in SlowAdcOpdfModel"""
     logd = -np.log(data)
-    return dot(logd * (1.5 - logd) * data, delta_q.T) - dot(data, delta_b.T)
+    return (np.dot(logd * (1.5 - logd) * data, delta_q.T)
+            - np.dot(data, delta_b.T))
 
 
 class QballModel(QballBaseModel):
@@ -703,21 +866,21 @@ class QballModel(QballBaseModel):
 
     References
     ----------
-    .. [1] Descoteaux, M., et. al. 2007. Regularized, fast, and robust
+    .. [1] Descoteaux, M., et al. 2007. Regularized, fast, and robust
            analytical Q-ball imaging.
     """
 
     def _set_fit_matrix(self, B, L, F, smooth):
-        invB = smooth_pinv(B, sqrt(smooth) * L)
+        invB = smooth_pinv(B, np.sqrt(smooth) * L)
         F = F[:, None]
         self._fit_matrix = F * invB
 
     def _get_shm_coef(self, data, mask=None):
         """Returns the coefficients of the model"""
-        return dot(data[..., self._where_dwi], self._fit_matrix.T)
+        return np.dot(data[..., self._where_dwi], self._fit_matrix.T)
 
 
-def normalize_data(data, where_b0, min_signal=1., out=None):
+def normalize_data(data, where_b0, min_signal=1e-5, out=None):
     """Normalizes the data with respect to the mean b0
     """
     if out is None:
@@ -737,8 +900,8 @@ def hat(B):
     """Returns the hat matrix for the design matrix B
     """
 
-    U, S, V = svd(B, False)
-    H = dot(U, U.T)
+    U, S, V = np.linalg.svd(B, False)
+    H = np.dot(U, U.T)
     return H
 
 
@@ -752,9 +915,9 @@ def lcr_matrix(H):
     if H.ndim != 2 or H.shape[0] != H.shape[1]:
         raise ValueError('H should be a square matrix')
 
-    leverages = sqrt(1 - H.diagonal())
+    leverages = np.sqrt(1 - H.diagonal(), where=H.diagonal() <= 1)
     leverages = leverages[:, None]
-    R = (eye(len(H)) - H) / leverages
+    R = (np.eye(len(H)) - H) / leverages
     return R - R.mean(0)
 
 
@@ -784,7 +947,7 @@ def bootstrap_data_array(data, H, R, permute=None):
     assert R.shape == H.shape
     assert len(permute) == R.shape[-1]
     R = R[permute]
-    data = dot(data, (H + R).T)
+    data = np.dot(data, (H + R).T)
     return data
 
 
@@ -795,8 +958,8 @@ def bootstrap_data_voxel(data, H, R, permute=None):
     """
     if permute is None:
         permute = randint(data.shape[-1], size=data.shape[-1])
-    r = dot(data, R.T)
-    boot_data = dot(data, H.T)
+    r = np.dot(data, R.T)
+    boot_data = np.dot(data, H.T)
     boot_data += r[permute]
     return boot_data
 
@@ -809,7 +972,7 @@ class ResidualBootstrapWrapper(object):
     There wrapper than samples the residual boostrap distribution of signal and
     returns that sample.
     """
-    def __init__(self, signal_object, B, where_dwi, min_signal=1.):
+    def __init__(self, signal_object, B, where_dwi, min_signal=1e-5):
         """Builds a ResidualBootstrapWapper
 
         Given some linear model described by B, the design matrix, and a
@@ -860,12 +1023,19 @@ def sf_to_sh(sf, sphere, sh_order=4, basis_type=None, smooth=0.0):
         The points on which the sf is defined.
     sh_order : int, optional
         Maximum SH order in the SH fit.  For `sh_order`, there will be
-        ``(sh_order + 1) * (sh_order_2) / 2`` SH coefficients (default 4).
-    basis_type : {None, 'mrtrix', 'fibernav'}
-        ``None`` for the default dipy basis,
-        ``mrtrix`` for the MRtrix basis, and
-        ``fibernav`` for the FiberNavigator basis
-        (default ``None``).
+        ``(sh_order + 1) * (sh_order + 2) / 2`` SH coefficients for a symmetric
+        basis and ``(sh_order + 1) * (sh_order + 1)`` coefficients for a full
+        SH basis (default 4).
+    basis_type : {None, 'tournier07', 'descoteaux07',
+                  'tournier07_full', 'descoteaux07_full'}
+        ``None`` for the default DIPY basis,
+        ``tournier07`` for the symmetric Tournier 2007 [2]_ basis,
+        ``descoteaux07`` for the symmetric Descoteaux 2007 [1]_ basis,
+        ``tournier07_full`` for Tournier 2007 [2]_ basis with odd and\
+                            even order terms, and
+        ``descoteaux07_full`` for Descoteaux 2007 [1]_ basis with odd and\
+                              even order terms
+        (``None`` defaults to ``descoteaux07``).
     smooth : float, optional
         Lambda-regularization in the SH fit (default 0.0).
 
@@ -874,7 +1044,18 @@ def sf_to_sh(sf, sphere, sh_order=4, basis_type=None, smooth=0.0):
     sh : ndarray
         SH coefficients representing the input function.
 
+    References
+    ----------
+    .. [1] Descoteaux, M., Angelino, E., Fitzgibbons, S. and Deriche, R.
+           Regularized, Fast, and Robust Analytical Q-ball Imaging.
+           Magn. Reson. Med. 2007;58:497-510.
+    .. [2] Tournier J.D., Calamante F. and Connelly A. Robust determination
+           of the fibre orientation distribution in diffusion MRI:
+           Non-negativity constrained super-resolved spherical deconvolution.
+           NeuroImage. 2007;35(4):1459-1472.
+
     """
+
     sph_harm_basis = sph_harm_lookup.get(basis_type)
 
     if sph_harm_basis is None:
@@ -882,13 +1063,13 @@ def sf_to_sh(sf, sphere, sh_order=4, basis_type=None, smooth=0.0):
     B, m, n = sph_harm_basis(sh_order, sphere.theta, sphere.phi)
 
     L = -n * (n + 1)
-    invB = smooth_pinv(B, sqrt(smooth) * L)
+    invB = smooth_pinv(B, np.sqrt(smooth) * L)
     sh = np.dot(sf, invB.T)
 
     return sh
 
 
-def sh_to_sf(sh, sphere, sh_order, basis_type=None):
+def sh_to_sf(sh, sphere, sh_order=4, basis_type=None):
     """Spherical harmonics (SH) to spherical function (SF).
 
     Parameters
@@ -899,19 +1080,37 @@ def sh_to_sf(sh, sphere, sh_order, basis_type=None):
         The points on which to sample the spherical function.
     sh_order : int, optional
         Maximum SH order in the SH fit.  For `sh_order`, there will be
-        ``(sh_order + 1) * (sh_order_2) / 2`` SH coefficients (default 4).
-    basis_type : {None, 'mrtrix', 'fibernav'}
-        ``None`` for the default dipy basis,
-        ``mrtrix`` for the MRtrix basis, and
-        ``fibernav`` for the FiberNavigator basis
-        (default ``None``).
+        ``(sh_order + 1) * (sh_order + 2) / 2`` SH coefficients for a symmetric
+        basis and ``(sh_order + 1) * (sh_order + 1)`` coefficients for a full
+        SH basis (default 4).
+    basis_type : {None, 'tournier07', 'descoteaux07',
+                  'tournier07_full', 'descoteaux07_full'}
+        ``None`` for the default DIPY basis,
+        ``tournier07`` for the symmetric Tournier 2007 [2]_ basis,
+        ``descoteaux07`` for the symmetric Descoteaux 2007 [1]_ basis,
+        ``tournier07_full`` for Tournier 2007 [2]_ basis with odd and\
+                            even order terms, and
+        ``descoteaux07_full`` for Descoteaux 2007 [1]_ basis with odd and\
+                              even order terms
+        (``None`` defaults to ``descoteaux07``).
 
     Returns
     -------
     sf : ndarray
          Spherical function values on the `sphere`.
 
+    References
+    ----------
+    .. [1] Descoteaux, M., Angelino, E., Fitzgibbons, S. and Deriche, R.
+           Regularized, Fast, and Robust Analytical Q-ball Imaging.
+           Magn. Reson. Med. 2007;58:497-510.
+    .. [2] Tournier J.D., Calamante F. and Connelly A. Robust determination
+           of the fibre orientation distribution in diffusion MRI:
+           Non-negativity constrained super-resolved spherical deconvolution.
+           NeuroImage. 2007;35(4):1459-1472.
+
     """
+
     sph_harm_basis = sph_harm_lookup.get(basis_type)
 
     if sph_harm_basis is None:
@@ -923,7 +1122,7 @@ def sh_to_sf(sh, sphere, sh_order, basis_type=None):
     return sf
 
 
-def sh_to_sf_matrix(sphere, sh_order, basis_type=None, return_inv=True,
+def sh_to_sf_matrix(sphere, sh_order=4, basis_type=None, return_inv=True,
                     smooth=0):
     """ Matrix that transforms Spherical harmonics (SH) to spherical
     function (SF).
@@ -934,12 +1133,19 @@ def sh_to_sf_matrix(sphere, sh_order, basis_type=None, return_inv=True,
         The points on which to sample the spherical function.
     sh_order : int, optional
         Maximum SH order in the SH fit.  For `sh_order`, there will be
-        ``(sh_order + 1) * (sh_order_2) / 2`` SH coefficients (default 4).
-    basis_type : {None, 'mrtrix', 'fibernav'}
-        ``None`` for the default dipy basis,
-        ``mrtrix`` for the MRtrix basis, and
-        ``fibernav`` for the FiberNavigator basis
-        (default ``None``).
+        ``(sh_order + 1) * (sh_order + 2) / 2`` SH coefficients for a symmetric
+        basis and ``(sh_order + 1) * (sh_order + 1)`` coefficients for a full
+        SH basis (default 4).
+    basis_type : {None, 'tournier07', 'descoteaux07',
+                  'tournier07_full', 'descoteaux07_full'}
+        ``None`` for the default DIPY basis,
+        ``tournier07`` for the symmetric Tournier 2007 [2]_ basis,
+        ``descoteaux07`` for the symmetric Descoteaux 2007 [1]_ basis,
+        ``tournier07_full`` for Tournier 2007 [2]_ basis with odd and\
+                            even order terms, and
+        ``descoteaux07_full`` for Descoteaux 2007 [1]_ basis with odd and\
+                              even order terms
+        (``None`` defaults to ``descoteaux07``).
     return_inv : bool
         If True then the inverse of the matrix is also returned
     smooth : float, optional
@@ -953,7 +1159,18 @@ def sh_to_sf_matrix(sphere, sh_order, basis_type=None, return_inv=True,
     invB : ndarray
         Inverse of B.
 
+    References
+    ----------
+    .. [1] Descoteaux, M., Angelino, E., Fitzgibbons, S. and Deriche, R.
+           Regularized, Fast, and Robust Analytical Q-ball Imaging.
+           Magn. Reson. Med. 2007;58:497-510.
+    .. [2] Tournier J.D., Calamante F. and Connelly A. Robust determination
+           of the fibre orientation distribution in diffusion MRI:
+           Non-negativity constrained super-resolved spherical deconvolution.
+           NeuroImage. 2007;35(4):1459-1472.
+
     """
+
     sph_harm_basis = sph_harm_lookup.get(basis_type)
 
     if sph_harm_basis is None:
@@ -969,42 +1186,53 @@ def sh_to_sf_matrix(sphere, sh_order, basis_type=None, return_inv=True,
 
 
 def calculate_max_order(n_coeffs):
-        """Calculate the maximal harmonic order, given that you know the
-        number of parameters that were estimated.
+    r"""Calculate the maximal harmonic order, given that you know the
+    number of parameters that were estimated.
 
-        Parameters
-        ----------
-        n_coeffs : int
-            The number of SH coefficients
+    Parameters
+    ----------
+    n_coeffs : int
+        The number of SH coefficients
 
-        Returns
-        -------
-        L : int
-            The maximal SH order, given the number of coefficients
+    Returns
+    -------
+    L : int
+        The maximal SH order, given the number of coefficients
 
-        Notes
-        -----
-        The calculation in this function proceeds according to the following
-        logic:
-        .. math::
-           n = \frac{1}{2} (L+1) (L+2)
-           \rarrow 2n = L^2 + 3L + 2
-           \rarrow L^2 + 3L + 2 - 2n = 0
-           \rarrow L^2 + 3L + 2(1-n) = 0
-           \rarrow L_{1,2} = \frac{-3 \pm \sqrt{9 - 8 (1-n)}}{2}
-           \rarrow L{1,2} = \frac{-3 \pm \sqrt{1 + 8n}}{2}
+    Notes
+    -----
+    The calculation in this function proceeds according to the following
+    logic:
+    .. math::
+        n = \frac{1}{2} (L+1) (L+2)
+        \rarrow 2n = L^2 + 3L + 2
+        \rarrow L^2 + 3L + 2 - 2n = 0
+        \rarrow L^2 + 3L + 2(1-n) = 0
+        \rarrow L_{1,2} = \frac{-3 \pm \sqrt{9 - 8 (1-n)}}{2}
+        \rarrow L{1,2} = \frac{-3 \pm \sqrt{1 + 8n}}{2}
 
-        Finally, the positive value is chosen between the two options.
-        """
+    Finally, the positive value is chosen between the two options.
+    """
 
-        L1 = (-3 + np.sqrt(1 + 8 * n_coeffs)) / 2
-        L2 = (-3 - np.sqrt(1 + 8 * n_coeffs)) / 2
-        return np.int(max([L1, L2]))
+    # L2 is negative for all positive values of n_coeffs, so we don't
+    # bother even computing it:
+    # L2 = (-3 - np.sqrt(1 + 8 * n_coeffs)) / 2
+    # L1 is always the larger value, so we go with that:
+    L1 = (-3 + np.sqrt(1 + 8 * n_coeffs)) / 2.0
+    # Check that it is a whole even number:
+    if L1.is_integer() and not np.mod(L1, 2):
+        return int(L1)
+    else:
+        # Otherwise, the input didn't make sense:
+        raise ValueError("The input to ``calculate_max_order`` was ",
+                         "%s, but that is not a valid number" % n_coeffs,
+                         "of coefficients for a spherical harmonics ",
+                         "basis set.")
 
 
 def anisotropic_power(sh_coeffs, norm_factor=0.00001, power=2,
                       non_negative=True):
-    """Calculates anisotropic power map with a given SH coefficient matrix
+    r"""Calculate anisotropic power map with a given SH coefficient matrix.
 
     Parameters
     ----------
@@ -1025,7 +1253,7 @@ def anisotropic_power(sh_coeffs, norm_factor=0.00001, power=2,
         The log of the resulting power image.
 
     Notes
-    ----------
+    -----
     Calculate AP image based on a IxJxKxC SH coefficient matrix based on the
     equation:
     .. math::
@@ -1048,8 +1276,8 @@ def anisotropic_power(sh_coeffs, norm_factor=0.00001, power=2,
             anisotropy tissues from HARDI data,
             in: Proceedings of International Society for Magnetic Resonance in
             Medicine. Milan, Italy.
-    """
 
+    """
     dim = sh_coeffs.shape[:-1]
     n_coeffs = sh_coeffs.shape[-1]
     max_order = calculate_max_order(n_coeffs)
