@@ -1,15 +1,7 @@
-""" Testing DTI
-
-"""
-from __future__ import division, print_function, absolute_import
+"""Testing DTI."""
 
 import numpy as np
-from nose.tools import (assert_true, assert_equal,
-                        assert_almost_equal, assert_raises)
 import numpy.testing as npt
-from numpy.testing import (assert_array_equal, assert_array_almost_equal,
-                           assert_)
-import nibabel as nib
 
 import scipy.optimize as opt
 
@@ -23,7 +15,8 @@ from dipy.reconst.dti import (axial_diffusivity, color_fa,
                               _decompose_tensor_nan)
 
 from dipy.io.bvectxt import read_bvec_file
-from dipy.data import get_data, dsi_voxels, get_sphere
+from dipy.io.image import load_nifti_data
+from dipy.data import get_fnames, dsi_voxels, get_sphere
 
 from dipy.core.subdivide_octahedron import create_unit_sphere
 import dipy.core.gradients as grad
@@ -33,30 +26,25 @@ from dipy.sims.voxel import single_tensor
 
 
 def test_roll_evals():
-    """
-
-    """
     # Just making sure this never passes through
     weird_evals = np.array([1, 0.5])
     npt.assert_raises(ValueError, dti._roll_evals, weird_evals)
 
 
 def test_tensor_algebra():
-    """
-    Test that the computation of tensor determinant and norm is correct
-    """
+    # Test that the computation of tensor determinant and norm is correct
     test_arr = np.random.rand(10, 3, 3)
     t_det = dti.determinant(test_arr)
     t_norm = dti.norm(test_arr)
     for i, x in enumerate(test_arr):
-        assert_almost_equal(np.linalg.det(x), t_det[i])
-        assert_almost_equal(np.linalg.norm(x), t_norm[i])
+        npt.assert_almost_equal(np.linalg.det(x), t_det[i])
+        npt.assert_almost_equal(np.linalg.norm(x), t_norm[i])
 
 
 def test_odf_with_zeros():
-    fdata, fbval, fbvec = get_data('small_25')
+    fdata, fbval, fbvec = get_fnames('small_25')
     gtab = grad.gradient_table(fbval, fbvec)
-    data = nib.load(fdata).get_data()
+    data = load_nifti_data(fdata)
     dm = dti.TensorModel(gtab)
     df = dm.fit(data)
     df.evals[0, 0, 0] = np.array([0, 0, 0])
@@ -66,20 +54,20 @@ def test_odf_with_zeros():
 
 
 def test_tensor_model():
-    fdata, fbval, fbvec = get_data('small_25')
-    data1 = nib.load(fdata).get_data()
+    fdata, fbval, fbvec = get_fnames('small_25')
+    data1 = load_nifti_data(fdata)
     gtab1 = grad.gradient_table(fbval, fbvec)
     data2, gtab2 = dsi_voxels()
     for data, gtab in zip([data1, data2], [gtab1, gtab2]):
         dm = dti.TensorModel(gtab, 'LS')
         dtifit = dm.fit(data[0, 0, 0])
-        assert_equal(dtifit.fa < 0.9, True)
+        npt.assert_equal(dtifit.fa < 0.9, True)
         dm = dti.TensorModel(gtab, 'WLS')
         dtifit = dm.fit(data[0, 0, 0])
-        assert_equal(dtifit.fa < 0.9, True)
-        assert_equal(dtifit.fa > 0, True)
+        npt.assert_equal(dtifit.fa < 0.9, True)
+        npt.assert_equal(dtifit.fa > 0, True)
         sphere = create_unit_sphere(4)
-        assert_equal(len(dtifit.odf(sphere)), len(sphere.vertices))
+        npt.assert_equal(len(dtifit.odf(sphere)), len(sphere.vertices))
         # Check that the multivoxel case works:
         dtifit = dm.fit(data)
 
@@ -94,22 +82,22 @@ def test_tensor_model():
                                     decimal=3)
 
     # And smoke-test that all these operations return sensibly-shaped arrays:
-    assert_equal(dtifit.fa.shape, data.shape[:3])
-    assert_equal(dtifit.ad.shape, data.shape[:3])
-    assert_equal(dtifit.md.shape, data.shape[:3])
-    assert_equal(dtifit.rd.shape, data.shape[:3])
-    assert_equal(dtifit.trace.shape, data.shape[:3])
-    assert_equal(dtifit.mode.shape, data.shape[:3])
-    assert_equal(dtifit.linearity.shape, data.shape[:3])
-    assert_equal(dtifit.planarity.shape, data.shape[:3])
-    assert_equal(dtifit.sphericity.shape, data.shape[:3])
+    npt.assert_equal(dtifit.fa.shape, data.shape[:3])
+    npt.assert_equal(dtifit.ad.shape, data.shape[:3])
+    npt.assert_equal(dtifit.md.shape, data.shape[:3])
+    npt.assert_equal(dtifit.rd.shape, data.shape[:3])
+    npt.assert_equal(dtifit.trace.shape, data.shape[:3])
+    npt.assert_equal(dtifit.mode.shape, data.shape[:3])
+    npt.assert_equal(dtifit.linearity.shape, data.shape[:3])
+    npt.assert_equal(dtifit.planarity.shape, data.shape[:3])
+    npt.assert_equal(dtifit.sphericity.shape, data.shape[:3])
 
     # Test for the shape of the mask
-    assert_raises(ValueError, dm.fit, np.ones((10, 10, 3)), np.ones((3, 3)))
+    npt.assert_raises(ValueError, dm.fit, np.ones((10, 10, 3)), np.ones((3, 3)))
 
     # Make some synthetic data
     b0 = 1000.
-    bvecs, bvals = read_bvec_file(get_data('55dir_grad.bvec'))
+    bvecs, bvals = read_bvec_file(get_fnames('55dir_grad.bvec'))
     gtab = grad.gradient_table_from_bvals_bvecs(bvals, bvecs.T)
     # The first b value is 0., so we take the second one:
     B = bvals[1]
@@ -127,13 +115,13 @@ def test_tensor_model():
     # Check that eigenvalues and eigenvectors are properly sorted through
     # that previous operation:
     for i in range(3):
-        assert_array_almost_equal(np.dot(tensor, evecs[:, i]),
-                                  evals[i] * evecs[:, i])
+        npt.assert_array_almost_equal(np.dot(tensor, evecs[:, i]),
+                                      evals[i] * evecs[:, i])
     # Design Matrix
     X = dti.design_matrix(gtab)
     # Signals
     Y = np.exp(np.dot(X, D))
-    assert_almost_equal(Y[0], b0)
+    npt.assert_almost_equal(Y[0], b0)
     Y.shape = (-1,) + Y.shape
 
     # Test fitting with different methods:
@@ -143,10 +131,10 @@ def test_tensor_model():
                                        return_S0_hat=True)
 
         tensor_fit = tensor_model.fit(Y)
-        assert_true(tensor_fit.model is tensor_model)
-        assert_equal(tensor_fit.shape, Y.shape[:-1])
-        assert_array_almost_equal(tensor_fit.evals[0], evals)
-        assert_array_almost_equal(tensor_fit.S0_hat, b0, decimal=3)
+        assert tensor_fit.model is tensor_model
+        npt.assert_equal(tensor_fit.shape, Y.shape[:-1])
+        npt.assert_array_almost_equal(tensor_fit.evals[0], evals)
+        npt.assert_array_almost_equal(tensor_fit.S0_hat, b0, decimal=3)
         # Test that the eigenvectors are correct, one-by-one:
         for i in range(3):
             # Eigenvectors have intrinsic sign ambiguity
@@ -154,28 +142,27 @@ def test_tensor_model():
             # http://prod.sandia.gov/techlib/access-control.cgi/2007/076422.pdf)
             # so we need to allow for sign flips. One of the following should
             # always be true:
-            assert_(
-                    np.all(np.abs(tensor_fit.evecs[0][:, i] -
-                                  evecs[:, i]) < 10e-6) or
-                    np.all(np.abs(-tensor_fit.evecs[0][:, i] -
-                                  evecs[:, i]) < 10e-6))
+            npt.assert_(np.all(np.abs(tensor_fit.evecs[0][:, i] -
+                                      evecs[:, i]) < 10e-6) or
+                        np.all(np.abs(-tensor_fit.evecs[0][:, i] -
+                                      evecs[:, i]) < 10e-6))
             # We set a fixed tolerance of 10e-6, similar to array_almost_equal
 
         err_msg = "Calculation of tensor from Y does not compare to "
         err_msg += "analytical solution"
-        assert_array_almost_equal(tensor_fit.quadratic_form[0], tensor,
-                                  err_msg=err_msg)
+        npt.assert_array_almost_equal(tensor_fit.quadratic_form[0], tensor,
+                                      err_msg=err_msg)
 
-        assert_almost_equal(tensor_fit.md[0], md)
-        assert_array_almost_equal(tensor_fit.mode, mode, decimal=5)
-        assert_equal(tensor_fit.directions.shape[-2], 1)
-        assert_equal(tensor_fit.directions.shape[-1], 3)
+        npt.assert_almost_equal(tensor_fit.md[0], md)
+        npt.assert_array_almost_equal(tensor_fit.mode, mode, decimal=5)
+        npt.assert_equal(tensor_fit.directions.shape[-2], 1)
+        npt.assert_equal(tensor_fit.directions.shape[-1], 3)
 
     # Test error-handling:
-    assert_raises(ValueError,
-                  dti.TensorModel,
-                  gtab,
-                  fit_method='crazy_method')
+    npt.assert_raises(ValueError,
+                      dti.TensorModel,
+                      gtab,
+                      fit_method='crazy_method')
 
     # Test custom fit tensor method
     try:
@@ -197,19 +184,19 @@ def test_tensor_model():
 
     tensor_model = dti.TensorModel(gtab)
     fit = tensor_model.fit(data)
-    assert_array_almost_equal(fit[0].evals, evals)
+    npt.assert_array_almost_equal(fit[0].evals, evals)
 
     # Return S0_test
     tensor_model = dti.TensorModel(gtab, return_S0_hat=True)
     fit = tensor_model.fit(data)
-    assert_array_almost_equal(fit[0].evals, evals)
-    assert_array_almost_equal(fit[0].S0_hat, b0)
+    npt.assert_array_almost_equal(fit[0].evals, evals)
+    npt.assert_array_almost_equal(fit[0].S0_hat, b0)
 
     # Evals should be high for high diffusion voxel
-    assert_(all(fit[1].evals > evals[0] * .9))
+    assert all(fit[1].evals > evals[0] * .9)
 
     # Evals should be zero where data is masked
-    assert_array_almost_equal(fit[2].evals, 0.)
+    npt.assert_array_almost_equal(fit[2].evals, 0.)
 
 
 def test_indexing_on_tensor_fit():
@@ -217,31 +204,31 @@ def test_indexing_on_tensor_fit():
     fit = dti.TensorFit(None, params)
 
     # Should return a TensorFit of appropriate shape
-    assert_equal(fit.shape, (2, 3, 4))
+    npt.assert_equal(fit.shape, (2, 3, 4))
     fit1 = fit[0]
-    assert_equal(fit1.shape, (3, 4))
-    assert_equal(type(fit1), dti.TensorFit)
+    npt.assert_equal(fit1.shape, (3, 4))
+    npt.assert_equal(type(fit1), dti.TensorFit)
     fit1 = fit[0, 0, 0]
-    assert_equal(fit1.shape, ())
-    assert_equal(type(fit1), dti.TensorFit)
+    npt.assert_equal(fit1.shape, ())
+    npt.assert_equal(type(fit1), dti.TensorFit)
     fit1 = fit[[0], slice(None)]
-    assert_equal(fit1.shape, (1, 3, 4))
-    assert_equal(type(fit1), dti.TensorFit)
+    npt.assert_equal(fit1.shape, (1, 3, 4))
+    npt.assert_equal(type(fit1), dti.TensorFit)
 
     # Should raise an index error if too many indices are passed
-    assert_raises(IndexError, fit.__getitem__, (0, 0, 0, 0))
+    npt.assert_raises(IndexError, fit.__getitem__, (0, 0, 0, 0))
 
 
 def test_fa_of_zero():
     evals = np.zeros((4, 3))
     fa = fractional_anisotropy(evals)
-    assert_array_equal(fa, 0)
+    npt.assert_array_equal(fa, 0)
 
 
 def test_ga_of_zero():
     evals = np.zeros((4, 3))
     ga = geodesic_anisotropy(evals)
-    assert_array_equal(ga, 0)
+    npt.assert_array_equal(ga, 0)
 
 
 def test_diffusivities():
@@ -266,13 +253,13 @@ def test_diffusivities():
     plan = planarity(dmfit.evals)
     spher = sphericity(dmfit.evals)
 
-    assert_almost_equal(md, (0.0015 + 0.0003 + 0.0001) / 3)
-    assert_almost_equal(Trace, (0.0015 + 0.0003 + 0.0001))
-    assert_almost_equal(ad, 0.0015)
-    assert_almost_equal(rd, (0.0003 + 0.0001) / 2)
-    assert_almost_equal(lin, (0.0015 - 0.0003)/Trace)
-    assert_almost_equal(plan, 2 * (0.0003 - 0.0001)/Trace)
-    assert_almost_equal(spher, (3 * 0.0001)/Trace)
+    npt.assert_almost_equal(md, (0.0015 + 0.0003 + 0.0001) / 3)
+    npt.assert_almost_equal(Trace, (0.0015 + 0.0003 + 0.0001))
+    npt.assert_almost_equal(ad, 0.0015)
+    npt.assert_almost_equal(rd, (0.0003 + 0.0001) / 2)
+    npt.assert_almost_equal(lin, (0.0015 - 0.0003)/Trace)
+    npt.assert_almost_equal(plan, 2 * (0.0003 - 0.0001)/Trace)
+    npt.assert_almost_equal(spher, (3 * 0.0001)/Trace)
 
 
 def test_color_fa():
@@ -289,8 +276,8 @@ def test_color_fa():
     evecs = np.zeros(fa.shape + (3, 3))
     evecs[..., :, :] = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]])
 
-    assert_equal(fa.shape, evecs[..., 0, 0].shape)
-    assert_equal((3, 3), evecs.shape[-2:])
+    npt.assert_equal(fa.shape, evecs[..., 0, 0].shape)
+    npt.assert_equal((3, 3), evecs.shape[-2:])
 
     # 3D test case
     fa = np.ones((3, 3, 3))
@@ -300,7 +287,7 @@ def test_color_fa():
     cfa_truth = np.array([1, 0, 0])
     true_cfa = np.reshape(np.tile(cfa_truth, 27), [3, 3, 3, 3])
 
-    assert_array_equal(cfa, true_cfa)
+    npt.assert_array_equal(cfa, true_cfa)
 
     # 2D test case
     fa = np.ones((3, 3))
@@ -310,7 +297,7 @@ def test_color_fa():
     cfa_truth = np.array([1, 0, 0])
     true_cfa = np.reshape(np.tile(cfa_truth, 9), [3, 3, 3])
 
-    assert_array_equal(cfa, true_cfa)
+    npt.assert_array_equal(cfa, true_cfa)
 
     # 1D test case
     fa = np.ones((3))
@@ -320,7 +307,7 @@ def test_color_fa():
     cfa_truth = np.array([1, 0, 0])
     true_cfa = np.reshape(np.tile(cfa_truth, 3), [3, 3])
 
-    assert_array_equal(cfa, true_cfa)
+    npt.assert_array_equal(cfa, true_cfa)
 
 
 def test_wls_and_ls_fit():
@@ -337,7 +324,7 @@ def test_wls_and_ls_fit():
 
     # Recall: D = [Dxx,Dyy,Dzz,Dxy,Dxz,Dyz,log(S_0)] and D ~ 10^-4 mm^2 /s
     b0 = 1000.
-    bvec, bval = read_bvec_file(get_data('55dir_grad.bvec'))
+    bvec, bval = read_bvec_file(get_fnames('55dir_grad.bvec'))
     B = bval[1]
     # Scale the eigenvalues and tensor by the B value so the units match
     D = np.array([1., 1., 1., 0., 0., 1., -np.log(b0) * B]) / B
@@ -349,10 +336,10 @@ def test_wls_and_ls_fit():
     X = dti.design_matrix(gtab)
     # Signals
     Y = np.exp(np.dot(X, D))
-    assert_almost_equal(Y[0], b0)
+    npt.assert_almost_equal(Y[0], b0)
     Y.shape = (-1,) + Y.shape
 
-    # Testing WLS Fit on Single Voxel
+    # Testing WLS Fit on single voxel
     # If you do something wonky (passing min_signal<0), you should get an
     # error:
     npt.assert_raises(ValueError, TensorModel, gtab, fit_method='WLS',
@@ -361,34 +348,34 @@ def test_wls_and_ls_fit():
     # Estimate tensor from test signals
     model = TensorModel(gtab, fit_method='WLS', return_S0_hat=True)
     tensor_est = model.fit(Y)
-    assert_equal(tensor_est.shape, Y.shape[:-1])
-    assert_array_almost_equal(tensor_est.evals[0], evals)
-    assert_array_almost_equal(tensor_est.quadratic_form[0], tensor,
-                              err_msg="Calculation of tensor from Y does not "
-                                      "compare to analytical solution")
-    assert_almost_equal(tensor_est.md[0], md)
-    assert_array_almost_equal(tensor_est.S0_hat[0], b0, decimal=3)
+    npt.assert_equal(tensor_est.shape, Y.shape[:-1])
+    npt.assert_array_almost_equal(tensor_est.evals[0], evals)
+    npt.assert_array_almost_equal(tensor_est.quadratic_form[0], tensor,
+                                  err_msg="Calculation of tensor from Y does "
+                                          "not compare to analytical solution")
+    npt.assert_almost_equal(tensor_est.md[0], md)
+    npt.assert_array_almost_equal(tensor_est.S0_hat[0], b0, decimal=3)
 
     # Test that we can fit a single voxel's worth of data (a 1d array)
     y = Y[0]
     tensor_est = model.fit(y)
-    assert_equal(tensor_est.shape, tuple())
-    assert_array_almost_equal(tensor_est.evals, evals)
-    assert_array_almost_equal(tensor_est.quadratic_form, tensor)
-    assert_almost_equal(tensor_est.md, md)
-    assert_array_almost_equal(tensor_est.lower_triangular(b0), D)
+    npt.assert_equal(tensor_est.shape, tuple())
+    npt.assert_array_almost_equal(tensor_est.evals, evals)
+    npt.assert_array_almost_equal(tensor_est.quadratic_form, tensor)
+    npt.assert_almost_equal(tensor_est.md, md)
+    npt.assert_array_almost_equal(tensor_est.lower_triangular(b0), D)
 
     # Test using fit_method='LS'
     model = TensorModel(gtab, fit_method='LS')
     tensor_est = model.fit(y)
-    assert_equal(tensor_est.shape, tuple())
-    assert_array_almost_equal(tensor_est.evals, evals)
-    assert_array_almost_equal(tensor_est.quadratic_form, tensor)
-    assert_almost_equal(tensor_est.md, md)
-    assert_array_almost_equal(tensor_est.lower_triangular(b0), D)
-    assert_array_almost_equal(tensor_est.linearity, linearity(evals))
-    assert_array_almost_equal(tensor_est.planarity, planarity(evals))
-    assert_array_almost_equal(tensor_est.sphericity, sphericity(evals))
+    npt.assert_equal(tensor_est.shape, tuple())
+    npt.assert_array_almost_equal(tensor_est.evals, evals)
+    npt.assert_array_almost_equal(tensor_est.quadratic_form, tensor)
+    npt.assert_almost_equal(tensor_est.md, md)
+    npt.assert_array_almost_equal(tensor_est.lower_triangular(b0), D)
+    npt.assert_array_almost_equal(tensor_est.linearity, linearity(evals))
+    npt.assert_array_almost_equal(tensor_est.planarity, planarity(evals))
+    npt.assert_array_almost_equal(tensor_est.sphericity, sphericity(evals))
 
 
 def test_masked_array_with_tensor():
@@ -396,60 +383,60 @@ def test_masked_array_with_tensor():
     mask = np.array([[True, False, False, True],
                      [True, False, True, False]])
 
-    bvec, bval = read_bvec_file(get_data('55dir_grad.bvec'))
+    bvec, bval = read_bvec_file(get_fnames('55dir_grad.bvec'))
     gtab = grad.gradient_table_from_bvals_bvecs(bval, bvec.T)
 
     tensor_model = TensorModel(gtab)
     tensor = tensor_model.fit(data, mask=mask)
-    assert_equal(tensor.shape, (2, 4))
-    assert_equal(tensor.fa.shape, (2, 4))
-    assert_equal(tensor.evals.shape, (2, 4, 3))
-    assert_equal(tensor.evecs.shape, (2, 4, 3, 3))
+    npt.assert_equal(tensor.shape, (2, 4))
+    npt.assert_equal(tensor.fa.shape, (2, 4))
+    npt.assert_equal(tensor.evals.shape, (2, 4, 3))
+    npt.assert_equal(tensor.evecs.shape, (2, 4, 3, 3))
 
     tensor = tensor[0]
-    assert_equal(tensor.shape, (4,))
-    assert_equal(tensor.fa.shape, (4,))
-    assert_equal(tensor.evals.shape, (4, 3))
-    assert_equal(tensor.evecs.shape, (4, 3, 3))
+    npt.assert_equal(tensor.shape, (4,))
+    npt.assert_equal(tensor.fa.shape, (4,))
+    npt.assert_equal(tensor.evals.shape, (4, 3))
+    npt.assert_equal(tensor.evecs.shape, (4, 3, 3))
 
     tensor = tensor[0]
-    assert_equal(tensor.shape, tuple())
-    assert_equal(tensor.fa.shape, tuple())
-    assert_equal(tensor.evals.shape, (3,))
-    assert_equal(tensor.evecs.shape, (3, 3))
-    assert_equal(type(tensor.model_params), np.ndarray)
+    npt.assert_equal(tensor.shape, tuple())
+    npt.assert_equal(tensor.fa.shape, tuple())
+    npt.assert_equal(tensor.evals.shape, (3,))
+    npt.assert_equal(tensor.evecs.shape, (3, 3))
+    npt.assert_equal(type(tensor.model_params), np.ndarray)
 
 
 def test_fit_method_error():
-    bvec, bval = read_bvec_file(get_data('55dir_grad.bvec'))
+    bvec, bval = read_bvec_file(get_fnames('55dir_grad.bvec'))
     gtab = grad.gradient_table_from_bvals_bvecs(bval, bvec.T)
 
     # This should work (smoke-testing!):
     TensorModel(gtab, fit_method='WLS')
 
     # This should raise an error because there is no such fit_method
-    assert_raises(ValueError, TensorModel, gtab, min_signal=1e-9,
-                  fit_method='s')
+    npt.assert_raises(ValueError, TensorModel, gtab, min_signal=1e-9,
+                      fit_method='s')
 
 
 def test_lower_triangular():
     tensor = np.arange(9).reshape((3, 3))
     D = lower_triangular(tensor)
-    assert_array_equal(D, [0, 3, 4, 6, 7, 8])
+    npt.assert_array_equal(D, [0, 3, 4, 6, 7, 8])
     D = lower_triangular(tensor, 1)
-    assert_array_equal(D, [0, 3, 4, 6, 7, 8, 0])
-    assert_raises(ValueError, lower_triangular, np.zeros((2, 3)))
+    npt.assert_array_equal(D, [0, 3, 4, 6, 7, 8, 0])
+    npt.assert_raises(ValueError, lower_triangular, np.zeros((2, 3)))
     shape = (4, 5, 6)
     many_tensors = np.empty(shape + (3, 3))
     many_tensors[:] = tensor
     result = np.empty(shape + (6,))
     result[:] = [0, 3, 4, 6, 7, 8]
     D = lower_triangular(many_tensors)
-    assert_array_equal(D, result)
+    npt.assert_array_equal(D, result)
     D = lower_triangular(many_tensors, 1)
     result = np.empty(shape + (7,))
     result[:] = [0, 3, 4, 6, 7, 8, 0]
-    assert_array_equal(D, result)
+    npt.assert_array_equal(D, result)
 
 
 def test_from_lower_triangular():
@@ -458,31 +445,31 @@ def test_from_lower_triangular():
                        [3, 4, 5]])
     D = np.arange(7)
     tensor = from_lower_triangular(D)
-    assert_array_equal(tensor, result)
+    npt.assert_array_equal(tensor, result)
     result = result * np.ones((5, 4, 1, 1))
     D = D * np.ones((5, 4, 1))
     tensor = from_lower_triangular(D)
-    assert_array_equal(tensor, result)
+    npt.assert_array_equal(tensor, result)
 
 
 def test_all_constant():
-    bvecs, bvals = read_bvec_file(get_data('55dir_grad.bvec'))
+    bvecs, bvals = read_bvec_file(get_fnames('55dir_grad.bvec'))
     gtab = grad.gradient_table_from_bvals_bvecs(bvals, bvecs.T)
     fit_methods = ['LS', 'OLS', 'NNLS', 'RESTORE']
     for _ in fit_methods:
         dm = dti.TensorModel(gtab)
-        assert_almost_equal(dm.fit(100 * np.ones(bvals.shape[0])).fa, 0)
+        npt.assert_almost_equal(dm.fit(100 * np.ones(bvals.shape[0])).fa, 0)
         # Doesn't matter if the signal is smaller than 1:
-        assert_almost_equal(dm.fit(0.4 * np.ones(bvals.shape[0])).fa, 0)
+        npt.assert_almost_equal(dm.fit(0.4 * np.ones(bvals.shape[0])).fa, 0)
 
 
 def test_all_zeros():
-    bvecs, bvals = read_bvec_file(get_data('55dir_grad.bvec'))
+    bvecs, bvals = read_bvec_file(get_fnames('55dir_grad.bvec'))
     gtab = grad.gradient_table_from_bvals_bvecs(bvals, bvecs.T)
     fit_methods = ['LS', 'OLS', 'NNLS', 'RESTORE']
     for _ in fit_methods:
         dm = dti.TensorModel(gtab)
-        assert_array_almost_equal(dm.fit(np.zeros(bvals.shape[0])).evals, 0)
+        npt.assert_array_almost_equal(dm.fit(np.zeros(bvals.shape[0])).evals, 0)
 
 
 def test_mask():
@@ -493,13 +480,13 @@ def test_mask():
     dtifit = dm.fit(data)
     dtifit_w_mask = dm.fit(data, mask=mask)
     # Without a mask it has some value
-    assert_(not np.isnan(dtifit.fa[0, 0, 0]))
+    assert not np.isnan(dtifit.fa[0, 0, 0])
     # Where mask is False, evals, evecs and fa should all be 0
-    assert_array_equal(dtifit_w_mask.evals[~mask], 0)
-    assert_array_equal(dtifit_w_mask.evecs[~mask], 0)
-    assert_array_equal(dtifit_w_mask.fa[~mask], 0)
+    npt.assert_array_equal(dtifit_w_mask.evals[~mask], 0)
+    npt.assert_array_equal(dtifit_w_mask.evecs[~mask], 0)
+    npt.assert_array_equal(dtifit_w_mask.fa[~mask], 0)
     # Except for the one voxel that was selected by the mask:
-    assert_almost_equal(dtifit_w_mask.fa[0, 0, 0], dtifit.fa[0, 0, 0])
+    npt.assert_almost_equal(dtifit_w_mask.fa[0, 0, 0], dtifit.fa[0, 0, 0])
 
     # Test with returning S0_hat
     dm = dti.TensorModel(gtab, 'LS', return_S0_hat=True)
@@ -511,21 +498,21 @@ def test_mask():
         dtifit = dm.fit(data)
         dtifit_w_mask = dm.fit(data, mask=mask)
         # Without a mask it has some value
-        assert_(not np.isnan(dtifit.fa[0, 0, 0]))
+        assert not np.isnan(dtifit.fa[0, 0, 0])
         # Where mask is False, evals, evecs and fa should all be 0
-        assert_array_equal(dtifit_w_mask.evals[~mask], 0)
-        assert_array_equal(dtifit_w_mask.evecs[~mask], 0)
-        assert_array_equal(dtifit_w_mask.fa[~mask], 0)
-        assert_array_equal(dtifit_w_mask.S0_hat[~mask], 0)
+        npt.assert_array_equal(dtifit_w_mask.evals[~mask], 0)
+        npt.assert_array_equal(dtifit_w_mask.evecs[~mask], 0)
+        npt.assert_array_equal(dtifit_w_mask.fa[~mask], 0)
+        npt.assert_array_equal(dtifit_w_mask.S0_hat[~mask], 0)
         # Except for the one voxel that was selected by the mask:
-        assert_almost_equal(dtifit_w_mask.fa[0, 0, 0], dtifit.fa[0, 0, 0])
-        assert_almost_equal(dtifit_w_mask.S0_hat[0, 0, 0],
-                            dtifit.S0_hat[0, 0, 0])
+        npt.assert_almost_equal(dtifit_w_mask.fa[0, 0, 0], dtifit.fa[0, 0, 0])
+        npt.assert_almost_equal(dtifit_w_mask.S0_hat[0, 0, 0],
+                                dtifit.S0_hat[0, 0, 0])
 
 
 def test_nnls_jacobian_fucn():
     b0 = 1000.
-    bvecs, bval = read_bvec_file(get_data('55dir_grad.bvec'))
+    bvecs, bval = read_bvec_file(get_fnames('55dir_grad.bvec'))
     gtab = grad.gradient_table(bval, bvecs)
     B = bval[1]
 
@@ -544,7 +531,7 @@ def test_nnls_jacobian_fucn():
     for i in range(len(X)):
         args = [X[i], Y[i]]
         approx = opt.approx_fprime(D, dti._nlls_err_func, 1e-8, *args)
-        assert_true(np.allclose(approx, analytical[i]))
+        assert np.allclose(approx, analytical[i])
 
     # Test Jacobian at zero
     D = np.zeros_like(D)
@@ -553,7 +540,7 @@ def test_nnls_jacobian_fucn():
     for i in range(len(X)):
         args = [X[i], Y[i]]
         approx = opt.approx_fprime(D, dti._nlls_err_func, 1e-8, *args)
-        assert_true(np.allclose(approx, analytical[i]))
+        assert np.allclose(approx, analytical[i])
 
 
 def test_nlls_fit_tensor():
@@ -562,7 +549,7 @@ def test_nlls_fit_tensor():
     """
 
     b0 = 1000.
-    bvecs, bval = read_bvec_file(get_data('55dir_grad.bvec'))
+    bvecs, bval = read_bvec_file(get_fnames('55dir_grad.bvec'))
     gtab = grad.gradient_table(bval, bvecs)
     B = bval[1]
 
@@ -583,41 +570,41 @@ def test_nlls_fit_tensor():
     # using non-linear least squares:
     tensor_model = dti.TensorModel(gtab, fit_method='NLLS')
     tensor_est = tensor_model.fit(Y)
-    assert_equal(tensor_est.shape, Y.shape[:-1])
-    assert_array_almost_equal(tensor_est.evals[0], evals)
-    assert_array_almost_equal(tensor_est.quadratic_form[0], tensor)
-    assert_almost_equal(tensor_est.md[0], md)
+    npt.assert_equal(tensor_est.shape, Y.shape[:-1])
+    npt.assert_array_almost_equal(tensor_est.evals[0], evals)
+    npt.assert_array_almost_equal(tensor_est.quadratic_form[0], tensor)
+    npt.assert_almost_equal(tensor_est.md[0], md)
 
     # You can also do this without the Jacobian (though it's slower):
     tensor_model = dti.TensorModel(gtab, fit_method='NLLS', jac=False)
     tensor_est = tensor_model.fit(Y)
-    assert_equal(tensor_est.shape, Y.shape[:-1])
-    assert_array_almost_equal(tensor_est.evals[0], evals)
-    assert_array_almost_equal(tensor_est.quadratic_form[0], tensor)
-    assert_almost_equal(tensor_est.md[0], md)
+    npt.assert_equal(tensor_est.shape, Y.shape[:-1])
+    npt.assert_array_almost_equal(tensor_est.evals[0], evals)
+    npt.assert_array_almost_equal(tensor_est.quadratic_form[0], tensor)
+    npt.assert_almost_equal(tensor_est.md[0], md)
 
     # Using the gmm weighting scheme:
     tensor_model = dti.TensorModel(gtab, fit_method='NLLS', weighting='gmm')
     tensor_est = tensor_model.fit(Y)
-    assert_equal(tensor_est.shape, Y.shape[:-1])
-    assert_array_almost_equal(tensor_est.evals[0], evals)
-    assert_array_almost_equal(tensor_est.quadratic_form[0], tensor)
-    assert_almost_equal(tensor_est.md[0], md)
+    npt.assert_equal(tensor_est.shape, Y.shape[:-1])
+    npt.assert_array_almost_equal(tensor_est.evals[0], evals)
+    npt.assert_array_almost_equal(tensor_est.quadratic_form[0], tensor)
+    npt.assert_almost_equal(tensor_est.md[0], md)
 
     # If you use sigma weighting, you'd better provide a sigma:
     tensor_model = dti.TensorModel(gtab, fit_method='NLLS', weighting='sigma')
     npt.assert_raises(ValueError, tensor_model.fit, Y)
 
     # Use NLLS with some actual 4D data:
-    data, bvals, bvecs = get_data('small_25')
+    data, bvals, bvecs = get_fnames('small_25')
     gtab = grad.gradient_table(bvals, bvecs)
     tm1 = dti.TensorModel(gtab, fit_method='NLLS')
-    dd = nib.load(data).get_data()
+    dd = load_nifti_data(data)
     tf1 = tm1.fit(dd)
     tm2 = dti.TensorModel(gtab)
     tf2 = tm2.fit(dd)
 
-    assert_array_almost_equal(tf1.fa, tf2.fa, decimal=1)
+    npt.assert_array_almost_equal(tf1.fa, tf2.fa, decimal=1)
 
 
 def test_restore():
@@ -625,7 +612,7 @@ def test_restore():
     Test the implementation of the RESTORE algorithm
     """
     b0 = 1000.
-    bvecs, bval = read_bvec_file(get_data('55dir_grad.bvec'))
+    bvecs, bval = read_bvec_file(get_fnames('55dir_grad.bvec'))
     gtab = grad.gradient_table(bval, bvecs)
     B = bval[1]
 
@@ -651,10 +638,10 @@ def test_restore():
                                                sigma=67.0)
 
                 tensor_est = tensor_model.fit(this_y)
-                assert_array_almost_equal(tensor_est.evals[0], evals,
-                                          decimal=3)
-                assert_array_almost_equal(tensor_est.quadratic_form[0], tensor,
-                                          decimal=3)
+                npt.assert_array_almost_equal(tensor_est.evals[0], evals,
+                                              decimal=3)
+                npt.assert_array_almost_equal(tensor_est.quadratic_form[0],
+                                              tensor, decimal=3)
 
     # If sigma is very small, it still needs to work:
     tensor_model = dti.TensorModel(gtab, fit_method='restore', sigma=0.0001)
@@ -664,7 +651,7 @@ def test_restore():
     tensor_model = dti.TensorModel(gtab, fit_method='restore', sigma=0.0001,
                                    return_S0_hat=True)
     tmf = tensor_model.fit(Y.copy())
-    assert_almost_equal(tmf[0].S0_hat, b0)
+    npt.assert_almost_equal(tmf[0].S0_hat, b0)
 
 
 def test_adc():
@@ -682,14 +669,14 @@ def test_adc():
 
     pdd0 = dtifit.evecs[0, 0, 0, 0]
     sphere_pdd0 = dps.Sphere(x=pdd0[0], y=pdd0[1], z=pdd0[2])
-    assert_array_almost_equal(dtifit.adc(sphere_pdd0)[0, 0, 0],
-                              dtifit.ad[0, 0, 0], decimal=5)
+    npt.assert_array_almost_equal(dtifit.adc(sphere_pdd0)[0, 0, 0],
+                                  dtifit.ad[0, 0, 0], decimal=5)
 
     # Test that it works for cases in which the data is 1D
     dtifit = dm.fit(data[0, 0, 0])
     sphere_pdd0 = dps.Sphere(x=pdd0[0], y=pdd0[1], z=pdd0[2])
-    assert_array_almost_equal(dtifit.adc(sphere_pdd0),
-                              dtifit.ad, decimal=5)
+    npt.assert_array_almost_equal(dtifit.adc(sphere_pdd0),
+                                  dtifit.ad, decimal=5)
 
 
 def test_predict():
@@ -708,12 +695,12 @@ def test_predict():
 
     dm = dti.TensorModel(gtab, 'LS', return_S0_hat=True)
     dmfit = dm.fit(S)
-    assert_array_almost_equal(dmfit.predict(gtab, S0=100), S)
-    assert_array_almost_equal(dmfit.predict(gtab), S)
-    assert_array_almost_equal(dm.predict(dmfit.model_params, S0=100), S)
+    npt.assert_array_almost_equal(dmfit.predict(gtab, S0=100), S)
+    npt.assert_array_almost_equal(dmfit.predict(gtab), S)
+    npt.assert_array_almost_equal(dm.predict(dmfit.model_params, S0=100), S)
 
-    fdata, fbvals, fbvecs = get_data()
-    data = nib.load(fdata).get_data()
+    fdata, fbvals, fbvecs = get_fnames()
+    data = load_nifti_data(fdata)
     # Make the data cube a bit larger:
     data = np.tile(data.T, 2).T
     gtab = grad.gradient_table(fbvals, fbvecs)
@@ -721,21 +708,21 @@ def test_predict():
     dtif = dtim.fit(data)
     S0 = np.mean(data[..., gtab.b0s_mask], -1)
     p = dtif.predict(gtab, S0)
-    assert_equal(p.shape, data.shape)
+    npt.assert_equal(p.shape, data.shape)
     # Predict using S0_hat:
     dtim = dti.TensorModel(gtab, return_S0_hat=True)
     dtif = dtim.fit(data)
     p = dtif.predict(gtab)
-    assert_equal(p.shape, data.shape)
+    npt.assert_equal(p.shape, data.shape)
     p = dtif.predict(gtab, S0)
-    assert_equal(p.shape, data.shape)
+    npt.assert_equal(p.shape, data.shape)
 
     # Test iter_fit_tensor with S0_hat
     dtim = dti.TensorModel(gtab, step=2, return_S0_hat=True)
     dtif = dtim.fit(data)
     S0 = np.mean(data[..., gtab.b0s_mask], -1)
     p = dtif.predict(gtab, S0)
-    assert_equal(p.shape, data.shape)
+    npt.assert_equal(p.shape, data.shape)
 
     # Use a smaller step in predicting:
 
@@ -743,17 +730,18 @@ def test_predict():
     dtif = dtim.fit(data)
     S0 = np.mean(data[..., gtab.b0s_mask], -1)
     p = dtif.predict(gtab, S0)
-    assert_equal(p.shape, data.shape)
+    npt.assert_equal(p.shape, data.shape)
     # And with a scalar S0:
     S0 = 1
     p = dtif.predict(gtab, S0)
-    assert_equal(p.shape, data.shape)
+    npt.assert_equal(p.shape, data.shape)
     # Assign the step through kwarg:
     p = dtif.predict(gtab, S0, step=1)
-    assert_equal(p.shape, data.shape)
+    npt.assert_equal(p.shape, data.shape)
     # And without S0:
     p = dtif.predict(gtab, step=1)
-    assert_equal(p.shape, data.shape)
+    npt.assert_equal(p.shape, data.shape)
+
 
 def test_eig_from_lo_tri():
     psphere = get_sphere('symmetric362')
@@ -771,18 +759,23 @@ def test_eig_from_lo_tri():
     dmfit = dm.fit(S)
 
     lo_tri = lower_triangular(dmfit.quadratic_form)
-    assert_array_almost_equal(dti.eig_from_lo_tri(lo_tri), dmfit.model_params)
+    npt.assert_array_almost_equal(dti.eig_from_lo_tri(lo_tri),
+                                  dmfit.model_params)
+
+
 
 def test_min_signal_alone():
-    fdata, fbvals, fbvecs = get_data()
-    data = nib.load(fdata).get_data()
+    fdata, fbvals, fbvecs = get_fnames()
+    data = load_nifti_data(fdata)
     gtab = grad.gradient_table(fbvals, fbvecs)
 
     idx = tuple(np.array(np.where(data == np.min(data)))[:-1, 0])
     ten_model = dti.TensorModel(gtab)
     fit_alone = ten_model.fit(data[idx])
     fit_together = ten_model.fit(data)
-    npt.assert_array_almost_equal(fit_together.model_params[idx], fit_alone.model_params, decimal=12)
+    npt.assert_array_almost_equal(fit_together.model_params[idx],
+                                  fit_alone.model_params, decimal=12)
+
 
 def test_decompose_tensor_nan():
     D_fine = np.array([1.7e-3, 0.0, 0.3e-3, 0.0, 0.0, 0.2e-3])
@@ -792,11 +785,11 @@ def test_decompose_tensor_nan():
     lref, vref = decompose_tensor(from_lower_triangular(D_fine))
     lfine, vfine = _decompose_tensor_nan(from_lower_triangular(D_fine),
                                          from_lower_triangular(D_alter))
-    assert_array_almost_equal(lfine, np.array([1.7e-3, 0.3e-3, 0.2e-3]))
-    assert_array_almost_equal(vfine, vref)
+    npt.assert_array_almost_equal(lfine, np.array([1.7e-3, 0.3e-3, 0.2e-3]))
+    npt.assert_array_almost_equal(vfine, vref)
 
     lref, vref = decompose_tensor(from_lower_triangular(D_alter))
     lalter, valter = _decompose_tensor_nan(from_lower_triangular(D_nan),
                                            from_lower_triangular(D_alter))
-    assert_array_almost_equal(lalter, np.array([1.6e-3, 0.4e-3, 0.3e-3]))
-    assert_array_almost_equal(valter, vref)
+    npt.assert_array_almost_equal(lalter, np.array([1.6e-3, 0.4e-3, 0.3e-3]))
+    npt.assert_array_almost_equal(valter, vref)
