@@ -1,28 +1,28 @@
+import platform
+import time
+from math import factorial
+
+from scipy.special import gamma
+import scipy.integrate as integrate
+import warnings
 import numpy as np
-from dipy.data import get_gtab_taiwan_dsi
 from numpy.testing import (assert_almost_equal,
                            assert_array_almost_equal,
-                           assert_equal,
+                           assert_equal, assert_,
                            run_module_suite,
                            assert_raises)
+import pytest
+from dipy.core.sphere_stats import angular_similarity
+from dipy.core.subdivide_octahedron import create_unit_sphere
+from dipy.data import get_gtab_taiwan_dsi, default_sphere
+from dipy.direction.peaks import peak_directions
 from dipy.reconst.mapmri import MapmriModel, mapmri_index_matrix
 from dipy.reconst import dti, mapmri
-from dipy.sims.voxel import (MultiTensor,
-                             multi_tensor_pdf,
-                             single_tensor,
-                             cylinders_and_ball_soderman)
-from scipy.special import gamma
-from math import factorial
-from dipy.data import get_sphere
-from dipy.sims.voxel import add_noise
-import scipy.integrate as integrate
-from dipy.core.sphere_stats import angular_similarity
-from dipy.direction.peaks import peak_directions
 from dipy.reconst.odf import gfa
 from dipy.reconst.tests.test_dsi import sticks_and_ball_dummies
-from dipy.core.subdivide_octahedron import create_unit_sphere
 from dipy.reconst.shm import sh_to_sf
-import time
+from dipy.sims.voxel import (multi_tensor, multi_tensor_pdf, add_noise,
+                             single_tensor, cylinders_and_ball_soderman)
 
 
 def int_func(n):
@@ -30,12 +30,13 @@ def int_func(n):
                                           np.sqrt(2**(n + 1) * factorial(n))))
     return f
 
+
 def generate_signal_crossing(gtab, lambda1, lambda2, lambda3, angle2=60):
     mevals = np.array(([lambda1, lambda2, lambda3],
                        [lambda1, lambda2, lambda3]))
     angl = [(0, 0), (angle2, 0)]
-    S, sticks = MultiTensor(gtab, mevals, S0=100.0, angles=angl,
-                            fractions=[50, 50], snr=None)
+    S, sticks = multi_tensor(gtab, mevals, S0=100.0, angles=angl,
+                             fractions=[50, 50], snr=None)
     return S, sticks
 
 
@@ -72,26 +73,29 @@ def test_orthogonality_basis_functions():
     # we already know the spherical harmonics are orthonormal
     # only check j>0, l=0 basis functions
 
-    int1 = integrate.quad(lambda q:
-                          mapmri.mapmri_isotropic_radial_signal_basis(
-                              1, 0, diffusivity, q) *
-                          mapmri.mapmri_isotropic_radial_signal_basis(
-                              2, 0, diffusivity, q) * q ** 2, qmin, qmax)[0]
-    int2 = integrate.quad(lambda q:
-                          mapmri.mapmri_isotropic_radial_signal_basis(
-                              2, 0, diffusivity, q) *
-                          mapmri.mapmri_isotropic_radial_signal_basis(
-                              3, 0, diffusivity, q) * q ** 2, qmin, qmax)[0]
-    int3 = integrate.quad(lambda q:
-                          mapmri.mapmri_isotropic_radial_signal_basis(
-                              3, 0, diffusivity, q) *
-                          mapmri.mapmri_isotropic_radial_signal_basis(
-                              4, 0, diffusivity, q) * q ** 2, qmin, qmax)[0]
-    int4 = integrate.quad(lambda q:
-                          mapmri.mapmri_isotropic_radial_signal_basis(
-                              4, 0, diffusivity, q) *
-                          mapmri.mapmri_isotropic_radial_signal_basis(
-                              5, 0, diffusivity, q) * q ** 2, qmin, qmax)[0]
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore",
+                                category=integrate.IntegrationWarning)
+        int1 = integrate.quad(
+            lambda q: mapmri.mapmri_isotropic_radial_signal_basis(
+                1, 0, diffusivity, q) *
+            mapmri.mapmri_isotropic_radial_signal_basis(
+                2, 0, diffusivity, q) * q ** 2, qmin, qmax)[0]
+        int2 = integrate.quad(
+            lambda q: mapmri.mapmri_isotropic_radial_signal_basis(
+                2, 0, diffusivity, q) *
+            mapmri.mapmri_isotropic_radial_signal_basis(
+                3, 0, diffusivity, q) * q ** 2, qmin, qmax)[0]
+        int3 = integrate.quad(
+            lambda q: mapmri.mapmri_isotropic_radial_signal_basis(
+                3, 0, diffusivity, q) *
+            mapmri.mapmri_isotropic_radial_signal_basis(
+                4, 0, diffusivity, q) * q ** 2, qmin, qmax)[0]
+        int4 = integrate.quad(
+            lambda q: mapmri.mapmri_isotropic_radial_signal_basis(
+                4, 0, diffusivity, q) *
+            mapmri.mapmri_isotropic_radial_signal_basis(
+                5, 0, diffusivity, q) * q ** 2, qmin, qmax)[0]
 
     # checking for first 5 basis functions if they are indeed orthogonal
     assert_almost_equal(int1, 0.)
@@ -244,7 +248,6 @@ def test_mapmri_signal_fitting(radial_order=6):
         assert_almost_equal(nmse_signal, 0.0, 2)
 
 
-
 def test_mapmri_isotropic_static_scale_factor(radial_order=6):
     gtab = get_gtab_taiwan_dsi()
     D = 0.7e-3
@@ -281,9 +284,16 @@ def test_mapmri_isotropic_static_scale_factor(radial_order=6):
     # test if indeed the scale factor is fixed now
     assert_equal(np.all(mapf_scale_stat_reg_stat.mu == mu),
                  True)
-    # test if computation time is shorter
-    assert_equal(time_scale_stat_reg_stat < time_scale_adapt_reg_stat,
-                 True)
+
+    # test if computation time is shorter (except on Windows):
+    if not platform.system() == "Windows":
+        assert_equal(time_scale_stat_reg_stat < time_scale_adapt_reg_stat,
+                     True,
+                     "mapf_scale_stat_reg_stat ({0}s) slower than "
+                     "mapf_scale_adapt_reg_stat ({1}s). It should be the"
+                     " opposite.".format(time_scale_stat_reg_stat,
+                                         time_scale_adapt_reg_stat))
+
     # check if the fitted signal is the same
     assert_almost_equal(mapf_scale_stat_reg_stat.fitted_signal(),
                         mapf_scale_adapt_reg_stat.fitted_signal())
@@ -312,7 +322,7 @@ def test_mapmri_pdf_integral_unity(radial_order=6):
     gtab = get_gtab_taiwan_dsi()
     l1, l2, l3 = [0.0015, 0.0003, 0.0003]
     S, _ = generate_signal_crossing(gtab, l1, l2, l3)
-    sphere = get_sphere('symmetric724')
+    sphere = default_sphere
     # test MAPMRI fitting
 
     mapm = MapmriModel(gtab, radial_order=radial_order,
@@ -409,9 +419,19 @@ def test_mapmri_metrics_anisotropic(radial_order=6):
     assert_almost_equal(mapfit.rtap(), rtap_gt, 5)
     assert_almost_equal(mapfit.rtpp(), rtpp_gt, 5)
     assert_almost_equal(mapfit.rtop(), rtop_gt, 5)
-    assert_almost_equal(mapfit.ng(), 0., 5)
-    assert_almost_equal(mapfit.ng_parallel(), 0., 5)
-    assert_almost_equal(mapfit.ng_perpendicular(), 0., 5)
+    with warnings.catch_warnings(record=True) as w:
+        ng = mapfit.ng()
+        ng_parallel = mapfit.ng_parallel()
+        ng_perpendicular = mapfit.ng_perpendicular()
+        assert_equal(len(w), 3)
+        for l_w in w:
+            assert_(issubclass(l_w.category, UserWarning))
+            assert_("model bval_threshold must be lower than 2000".lower()
+                    in str(l_w.message).lower())
+
+    assert_almost_equal(ng, 0., 5)
+    assert_almost_equal(ng_parallel, 0., 5)
+    assert_almost_equal(ng_perpendicular, 0., 5)
     assert_almost_equal(mapfit.msd(), msd_gt, 5)
     assert_almost_equal(mapfit.qiv(), qiv_gt, 5)
 
@@ -615,7 +635,7 @@ def test_estimate_radius_with_rtap(radius_gt=5e-3):
     assert_almost_equal(radius_estimated, radius_gt, 4)
 
 
-@np.testing.dec.skipif(not mapmri.have_cvxpy)
+@pytest.mark.skipif(not mapmri.have_cvxpy, reason="Requires CVXPY")
 def test_positivity_constraint(radial_order=6):
     gtab = get_gtab_taiwan_dsi()
     l1, l2, l3 = [0.0015, 0.0003, 0.0003]
@@ -768,8 +788,8 @@ def test_laplacian_regularization(radial_order=6):
 def test_mapmri_odf(radial_order=6):
     gtab = get_gtab_taiwan_dsi()
 
-    # load symmetric 724 sphere
-    sphere = get_sphere('symmetric724')
+    # load repulsion 724 sphere
+    sphere = default_sphere
 
     # load icosahedron sphere
     l1, l2, l3 = [0.0015, 0.0003, 0.0003]
@@ -778,7 +798,7 @@ def test_mapmri_odf(radial_order=6):
     mapmod = MapmriModel(gtab, radial_order=radial_order,
                          laplacian_regularization=True,
                          laplacian_weighting=0.01)
-    # symmetric724
+    # repulsion724
     sphere2 = create_unit_sphere(5)
     mapfit = mapmod.fit(data)
     odf = mapfit.odf(sphere)
