@@ -1,5 +1,5 @@
 import numpy as np
-from dipy.data import get_sphere, get_3shell_gtab, get_isbi2013_2shell_gtab
+from dipy.data import default_sphere, get_3shell_gtab, get_isbi2013_2shell_gtab
 from dipy.reconst.shore import ShoreModel
 from dipy.reconst.shm import sh_to_sf
 from dipy.direction.peaks import peak_directions
@@ -7,7 +7,7 @@ from dipy.reconst.odf import gfa
 from numpy.testing import (assert_equal,
                            assert_almost_equal,
                            run_module_suite)
-from dipy.sims.voxel import SticksAndBall
+from dipy.sims.voxel import sticks_and_ball
 from dipy.core.subdivide_octahedron import create_unit_sphere
 from dipy.core.sphere_stats import angular_similarity
 from dipy.reconst.tests.test_dsi import sticks_and_ball_dummies
@@ -16,17 +16,17 @@ from dipy.reconst.tests.test_dsi import sticks_and_ball_dummies
 def test_shore_odf():
     gtab = get_isbi2013_2shell_gtab()
 
-    # load symmetric 724 sphere
-    sphere = get_sphere('symmetric724')
+    # load repulsion 724 sphere
+    sphere = default_sphere
 
     # load icosahedron sphere
     sphere2 = create_unit_sphere(5)
-    data, golden_directions = SticksAndBall(gtab, d=0.0015,
-                                            S0=100, angles=[(0, 0), (90, 0)],
-                                            fractions=[50, 50], snr=None)
+    data, golden_directions = sticks_and_ball(gtab, d=0.0015, S0=100,
+                                              angles=[(0, 0), (90, 0)],
+                                              fractions=[50, 50], snr=None)
     asm = ShoreModel(gtab, radial_order=6,
                      zeta=700, lambdaN=1e-8, lambdaL=1e-8)
-    # symmetric724
+    # repulsion724
     asmfit = asm.fit(data)
     odf = asmfit.odf(sphere)
     odf_sh = asmfit.odf_sh()
