@@ -11,13 +11,11 @@ import scipy.sparse as sps
 import scipy.linalg as la
 
 from dipy.reconst.base import ReconstModel, ReconstFit
-from dipy.utils.six.moves import range
 from dipy.tracking.utils import unique_rows
 from dipy.tracking.streamline import transform_streamlines
 from dipy.tracking.vox2track import _voxel2streamline
 import dipy.data as dpd
 import dipy.core.optimize as opt
-from dipy.testing import setup_test
 
 
 def gradient(f):
@@ -50,8 +48,8 @@ def gradient(f):
            [ 2.,  2., -1.]]), array([[ 1. ,  2.5,  4. ],
            [ 1. ,  1. ,  1. ]])]
 
-    Note
-    ----
+    Notes
+    ------
     This is a simplified implementation of gradient that is part of numpy
     1.8. In order to mitigate the effects of changes added to this
     implementation in version 1.9 of numpy, we include this implementation
@@ -75,18 +73,19 @@ def gradient(f):
         slice1[axis] = slice(1, -1)
         slice2[axis] = slice(2, None)
         slice3[axis] = slice(None, -2)
+
         # 1D equivalent -- out[1:-1] = (f[2:] - f[:-2])/2.0
-        out[slice1] = (f[slice2] - f[slice3])/2.0
+        out[tuple(slice1)] = (f[tuple(slice2)] - f[tuple(slice3)])/2.0
         slice1[axis] = 0
         slice2[axis] = 1
         slice3[axis] = 0
         # 1D equivalent -- out[0] = (f[1] - f[0])
-        out[slice1] = (f[slice2] - f[slice3])
+        out[tuple(slice1)] = (f[tuple(slice2)] - f[tuple(slice3)])
         slice1[axis] = -1
         slice2[axis] = -1
         slice3[axis] = -2
         # 1D equivalent -- out[-1] = (f[-1] - f[-2])
-        out[slice1] = (f[slice2] - f[slice3])
+        out[tuple(slice1)] = (f[tuple(slice2)] - f[tuple(slice3)])
 
         # divide by step size
         outvals.append(out / dx[axis])
@@ -135,7 +134,7 @@ def grad_tensor(grad, evals):
 
     """
     # This is the rotation matrix from [1, 0, 0] to this gradient of the sl:
-    R = la.svd(np.matrix(grad), overwrite_a=True)[2]
+    R = la.svd([grad], overwrite_a=True)[2]
     # This is the 3 by 3 tensor after rotation:
     T = np.dot(np.dot(R, np.diag(evals)), R.T)
     return T
@@ -158,8 +157,8 @@ def streamline_tensors(streamline, evals=[0.001, 0, 0]):
     -------
     An n_nodes by 3 by 3 array with the tensor for each node in the fiber.
 
-    Note
-    ----
+    Notes
+    ------
     Estimates of the radial/axial diffusivities may rely on
     empirical measurements (for example, the AD in the Corpus Callosum), or
     may be based on a biophysical model of some kind.
@@ -207,6 +206,7 @@ class LifeSignalMaker(object):
     A class for generating signals from streamlines in an efficient and speedy
     manner.
     """
+
     def __init__(self, gtab, evals=[0.001, 0, 0], sphere=None):
         """
         Initialize a signal maker
@@ -218,7 +218,7 @@ class LifeSignalMaker(object):
         evals : list of 3 items
             The eigenvalues of the canonical tensor to use in calculating the
             signal.
-        n_points : `dipy.core.Sphere` class instance
+        sphere : `dipy.core.Sphere` class instance
             The discrete sphere to use as an approximation for the continuous
             sphere on which the signal is represented. If integer - we will use
             an instance of one of the symmetric spheres cached in
@@ -226,10 +226,7 @@ class LifeSignalMaker(object):
             provided, we will use this object. Default: the :mod:`dipy.data`
             symmetric sphere with 724 vertices
         """
-        if sphere is None:
-            self.sphere = dpd.get_sphere('symmetric724')
-        else:
-            self.sphere = sphere
+        self.sphere = sphere or dpd.default_sphere
 
         self.gtab = gtab
         self.evals = evals
@@ -265,8 +262,7 @@ class LifeSignalMaker(object):
         return sig_out
 
 
-def voxel2streamline(streamline, transformed=False, affine=None,
-                     unique_idx=None):
+def voxel2streamline(streamline, affine, unique_idx=None):
     """
     Maps voxels to streamlines and streamlines to voxels, for setting up
     the LiFE equations matrix
@@ -276,15 +272,9 @@ def voxel2streamline(streamline, transformed=False, affine=None,
     streamline : list
         A collection of streamlines, each n by 3, with n being the number of
         nodes in the fiber.
-
-    affine : 4 by 4 array (optional)
-       Defines the spatial transformation from streamline to data.
-       Default: np.eye(4)
-
-    transformed : bool (optional)
-        Whether the streamlines have been already transformed (in which case
-        they don't need to be transformed in here).
-
+    affine : array_like (4, 4)
+        The mapping from voxel coordinates to streamline points.
+        The voxel_to_rasmm matrix, typically from a NIFTI file.
     unique_idx : array (optional).
        The unique indices in the streamlines
 
@@ -299,12 +289,7 @@ def voxel2streamline(streamline, transformed=False, affine=None,
     this streamline passes through, which nodes of that streamline are in that
     voxel?
     """
-    if transformed:
-        transformed_streamline = streamline
-    else:
-        if affine is None:
-            affine = np.eye(4)
-        transformed_streamline = transform_streamlines(streamline, affine)
+    transformed_streamline = transform_streamlines(streamline, affine)
 
     if unique_idx is None:
         all_coords = np.concatenate(transformed_streamline)
@@ -327,6 +312,7 @@ class FiberModel(ReconstModel):
         B.A. (2014). Validation and statistical inference in living
         connectomes. Nature Methods.
     """
+
     def __init__(self, gtab):
         """
         Parameters
@@ -347,8 +333,9 @@ class FiberModel(ReconstModel):
         ----------
         streamline : list
             Streamlines, each is an array of shape (n, 3)
-        affine : 4 by 4 array
-            Mapping from the streamline coordinates to the data
+        affine : array_like (4, 4)
+            The mapping from voxel coordinates to streamline points.
+            The voxel_to_rasmm matrix, typically from a NIFTI file.
         evals : list (3 items, optional)
             The eigenvalues of the canonical tensor used as a response
             function. Default:[0.001, 0, 0].
@@ -365,8 +352,6 @@ class FiberModel(ReconstModel):
                                           evals=evals,
                                           sphere=sphere)
 
-        if affine is None:
-            affine = np.eye(4)
         streamline = transform_streamlines(streamline, affine)
         # Assign some local variables, for shorthand:
         all_coords = np.concatenate(streamline)
@@ -374,11 +359,11 @@ class FiberModel(ReconstModel):
         del all_coords
         # We only consider the diffusion-weighted signals:
         n_bvecs = self.gtab.bvals[~self.gtab.b0s_mask].shape[0]
-        v2f, v2fn = voxel2streamline(streamline, transformed=True,
-                                     affine=affine, unique_idx=vox_coords)
+        v2f, v2fn = voxel2streamline(streamline, np.eye(4),
+                                     unique_idx=vox_coords)
         # How many fibers in each voxel (this will determine how many
         # components are in the matrix):
-        n_unique_f = len(np.hstack(v2f.values()))
+        n_unique_f = len(np.hstack(list(v2f.values())))
         # Preallocate these, which will be used to generate the sparse
         # matrix:
         f_matrix_sig = np.zeros(n_unique_f * n_bvecs, dtype=np.float)
@@ -420,7 +405,7 @@ class FiberModel(ReconstModel):
         # Allocate the sparse matrix, using the more memory-efficient 'csr'
         # format:
         life_matrix = sps.csr_matrix((f_matrix_sig,
-                                     [f_matrix_row, f_matrix_col]))
+                                      [f_matrix_row, f_matrix_col]))
 
         return life_matrix, vox_coords
 
@@ -451,7 +436,7 @@ class FiberModel(ReconstModel):
         return (to_fit, weighted_signal, b0_signal, relative_signal, mean_sig,
                 vox_data)
 
-    def fit(self, data, streamline, affine=None, evals=[0.001, 0, 0],
+    def fit(self, data, streamline, affine, evals=[0.001, 0, 0],
             sphere=None):
         """
         Fit the LiFE FiberModel for data and a set of streamlines associated
@@ -460,19 +445,15 @@ class FiberModel(ReconstModel):
         Parameters
         ----------
         data : 4D array
-           Diffusion-weighted data
-
+            Diffusion-weighted data
         streamline : list
-           A bunch of streamlines
-
-        affine: 4 by 4 array (optional)
-           The affine to go from the streamline coordinates to the data
-           coordinates. Defaults to use `np.eye(4)`
-
+            A bunch of streamlines
+        affine : array_like (4, 4)
+            The mapping from voxel coordinates to streamline points.
+            The voxel_to_rasmm matrix, typically from a NIFTI file.
         evals : list (optional)
-           The eigenvalues of the tensor response function used in constructing
-           the model signal. Default: [0.001, 0, 0]
-
+            The eigenvalues of the tensor response function used in
+            constructing the model signal. Default: [0.001, 0, 0]
         sphere: `dipy.core.Sphere` instance, or False
             Whether to approximate (and cache) the signal on a discrete
             sphere. This may confer a significant speed-up in setting up the
@@ -486,6 +467,10 @@ class FiberModel(ReconstModel):
         """
         if affine is None:
             affine = np.eye(4)
+        sl_len = np.array([len(s) for s in streamline])
+        if np.any(sl_len < 2):
+            raise ValueError("Input contains streamlines with only one node."
+            " The LiFE model cannot be fit with these streamlines included.")
         life_matrix, vox_coords = \
             self.setup(streamline, affine, evals=evals, sphere=sphere)
         (to_fit, weighted_signal, b0_signal, relative_signal, mean_sig,
@@ -500,6 +485,7 @@ class FiberFit(ReconstFit):
     """
     A fit of the LiFE model to diffusion data
     """
+
     def __init__(self, fiber_model, life_matrix, vox_coords, to_fit, beta,
                  weighted_signal, b0_signal, relative_signal, mean_sig,
                  vox_data, streamline, affine, evals):
