@@ -1,12 +1,12 @@
-from __future__ import division, print_function, absolute_import
 
 import numpy as np
-import nose
-from nose.tools import (assert_true, assert_false, assert_equal,
-                        assert_almost_equal)
-from numpy.testing import assert_array_equal, assert_array_almost_equal
-from dipy.tracking import metrics as tm
+from dipy.testing import assert_true, assert_false
+from numpy.testing import (assert_array_equal, assert_array_almost_equal,
+                           assert_equal, assert_almost_equal)
 from dipy.tracking import distances as pf
+from dipy.tracking.streamline import set_number_of_points
+from dipy.data import get_fnames
+from dipy.io.streamline import load_tractogram
 
 
 def test_LSCv2():
@@ -19,12 +19,12 @@ def test_LSCv2():
     xyz6 = np.array([[0, 20, 0], [0, 40, 0], [300, 50, 0]], dtype='float32')
 
     T = [xyz1, xyz2, xyz3, xyz4, xyz5, xyz6]
-    C = pf.local_skeleton_clustering(T, 0.2)
+    pf.local_skeleton_clustering(T, 0.2)
 
     # print C
     # print len(C)
 
-    C2 = pf.local_skeleton_clustering_3pts(T, 0.2)
+    pf.local_skeleton_clustering_3pts(T, 0.2)
 
     # print C2
     # print len(C2)
@@ -61,16 +61,11 @@ def test_LSCv2():
     print(t2-t1)
     print(len(C5))
 
-    from dipy.data import get_data
-    from nibabel import trackvis as tv
-    try:
-        from dipy.viz import fvtk
-    except ImportError as e:
-        raise nose.plugins.skip.SkipTest(
-            'Fails to import dipy.viz due to %s' % str(e))
+    fname = get_fnames('fornix')
+    fornix = load_tractogram(fname, 'same',
+                             bbox_valid_check=False).streamlines
 
-    streams, hdr = tv.read(get_data('fornix'))
-    T3 = [tm.downsample(s[0], 6) for s in streams]
+    T3 = set_number_of_points(fornix, 6)
 
     print('lenT3', len(T3))
 
@@ -79,16 +74,20 @@ def test_LSCv2():
     print('lenC', len(C))
 
     """
+    try:
+        from dipy.viz import window, actor
+    except ImportError as e:
+        raise pytest.skip('Fails to import dipy.viz due to %s' % str(e))
 
-    r = fvtk.ren()
+    scene = window.Scene()
     colors = np.zeros((len(C), 3))
     for c in C:
         color = np.random.rand(3)
         for i in C[c]['indices']:
-            fvtk.add(r, fvtk.line(T3[i], color))
+            scene.add(actor.line(T3[i], color))
         colors[c] = color
-    fvtk.show(r)
-    fvtk.clear(r)
+    window.show(scene)
+    scene.clear()
     skeleton = []
 
     def width(w):
@@ -102,12 +101,12 @@ def test_LSCv2():
         bundle = [T3[i] for i in C[c]['indices']]
         si,s = pf.most_similar_track_mam(bundle, 'avg')
         skeleton.append(bundle[si])
-        fvtk.label(r,text = str(len(bundle)), pos=(bundle[si][-1]),
-                   scale=(2, 2, 2))
-        fvtk.add(r, fvtk.line(skeleton, colors, opacity=1,
-                              linewidth = width(len(bundle)/10.)))
+        actor.label(r,text = str(len(bundle)), pos=(bundle[si][-1]),
+                    scale=(2, 2, 2))
+        scene.add(actor.line(skeleton, colors, opacity=1,
+                         linewidth = width(len(bundle)/10.)))
 
-    fvtk.show(r)
+    window.show(scene)
 
     """
 
@@ -121,7 +120,7 @@ def test_bundles_distances_mam():
     tracksA = [xyz1A, xyz2A]
     tracksB = [xyz1B, xyz1A, xyz2A]
     for metric in ('avg', 'min', 'max'):
-        DM2 = pf.bundles_distances_mam(tracksA, tracksB, metric=metric)
+        pf.bundles_distances_mam(tracksA, tracksB, metric=metric)
 
 
 def test_bundles_distances_mdf():
@@ -132,7 +131,7 @@ def test_bundles_distances_mdf():
 
     tracksA = [xyz1A, xyz2A]
     tracksB = [xyz1B, xyz1A, xyz2A]
-    DM2 = pf.bundles_distances_mdf(tracksA, tracksB)
+    pf.bundles_distances_mdf(tracksA, tracksB)
 
     tracksA = [xyz1A, xyz1A]
     tracksB = [xyz1A, xyz1A]
@@ -266,7 +265,7 @@ def test_most_similar_mam():
     tracks = [xyz1, xyz2, xyz3]
     for metric in ('avg', 'min', 'max'):
         # pf should be much faster and the results equivalent
-        si2, s2 = pf.most_similar_track_mam(tracks, metric=metric)
+        pf.most_similar_track_mam(tracks, metric=metric)
 
 
 def test_cut_plane():
